using System;
using System.IO;
using System.Text;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Random;

namespace EvoAlg.Ue2
{
    public class Ue2_Population : VectorPopulation<DoubleVectorMinimize, double>
    {
        public readonly double min;
        public readonly double max;

        public Ue2_Population
            (double min, double max, int vectorLength, int μ, int λ,
            IndividualFactory<EvoLab.CoreApi.Individuals.DoubleVectorMinimize> allocateNewIndividual)
            : base(vectorLength, μ, λ, allocateNewIndividual)
        {
            this.min = min;
            this.max = max;
        }
    }

    #region Initialization

    [Author("Lieferumfang von EvoLab")]
    [Description("Initialisierungsmodul")]
    [Identifier("Ue2_Initialization")]

    [IntParameter("μ", 30)]
    [IntParameter("λ", 150)]
    
    [DoubleParameter("Bereichs Minimum", -100.0)]
    [DoubleParameter("Bereichs Maximum", 100.0)]

    [IntParameter("Laenge der Individuen", 25)]

    public class Ue2_Initialization : EAInitializationBase<DoubleVectorMinimize, Ue2_Population> 
    {
        public override Ue2_Population CreatePopulation(string name, IWorld world,
            ISimulationControl mySimControl)
        {
            if (world.GetLocalInteger(name, "μ") < 1)
                throw new ArgumentOutOfRangeException (
                    "μ ist kleiner als 1. Keine leere Population möglich.");

            if (world.GetLocalInteger(name, "λ") < 1)
                throw new ArgumentOutOfRangeException (
                    "λ ist kleiner als 0. Keine negativ-anzahlige Nachkommenschaft möglich.");

            if (world.GetLocalInteger(name, "Laenge der Individuen") < 0)
                throw new ArgumentOutOfRangeException (
                    "Laenge der Individuen darf nicht kleiner als 0 sein.");

            int μ = world.GetLocalInteger(name, "μ");
            int length = world.GetLocalInteger(name, "Laenge der Individuen");
            Ue2_Population population = new Ue2_Population (
                world.GetLocalDouble(name, "Bereichs Minimum"),
                world.GetLocalDouble(name, "Bereichs Maximum"),
                length,
                μ,
                world.GetLocalInteger(name, "λ"),
                (new DoubleVectorMinimizeFactory (length)));

            foreach (DoubleVectorMinimize ind in population.Pop) {
                EvoLab.CoreApi.Operators.Initialization.RandomizeDoubleArray (
                    world.RandomNumberGenerator, ind.vector,
                    world.GetLocalDouble(name,"Bereichs Minimum"),
                    world.GetLocalDouble(name, "Bereichs Maximum"));
            }
			
			return population;
        }
    }

    #endregion Initialization

    #region Fitness

    [Author("Lieferumfang von EvoLab")]
    [Description("Berechnet die Fitness für jedes Individuum")]
    [Identifier("Ue2_Fitness_Rastrigin")]
    public sealed class Ue2_Fitness_Rastrigin
    : EAFitness<DoubleVectorMinimize, Ue2_Population>
    {
        protected override void Evaluate (DoubleVectorMinimize ind)
        {
            double sum = 0.0f;
            for (int i = 0; i < ind.vector.Length; ++i) {
                sum += RastriginSummand (ind.vector[i]);
            }
            ind.Fitness = 10 * ind.vector.Length + sum;
        }

        private double RastriginSummand (double xi)
        {
            return xi * xi - 10 * System.Math.Cos(2 * System.Math.PI * xi);
        }
    }

    [Author("Lieferumfang von EvoLab")]
    [Description("Berechnet die Fitness für jedes Individuum")]
    [Identifier("Ue2_Fitness_Rosenbrock")]
    public sealed class Ue2_Fitness_Rosenbrock
    : EAFitness<DoubleVectorMinimize, Ue2_Population>
    {
        protected override void Evaluate(DoubleVectorMinimize ind)
        {
            double sum = 0.0f;
            for (int i = 0; i < ind.vector.Length - 1; ++i)
            {
                sum += RosenbrockSummand(ind.vector[i], ind.vector[i+1]);
            }
            ind.Fitness = sum;
        }

        private double RosenbrockSummand(double xi, double yi)
        {
            return 100.0d * (yi - xi * xi) * (yi - xi * xi) + (xi - 1) * (xi - 1);
        }
    }

    [Author("Lieferumfang von EvoLab")]
    [Description("Berechnet die Fitness für jedes Individuum")]
    [Identifier("Ue2_Fitness_Djong")]
    public sealed class Ue2_Fitness_Djong
    : EAFitness<DoubleVectorMinimize, Ue2_Population>
    {
        protected override void Evaluate (DoubleVectorMinimize ind)
        {
            double sum = 0.0f;
            for (int i = 0; i < ind.vector.Length; ++i) {
                sum += DjongSummand (ind.vector[i], i+1);
            }
            ind.Fitness = sum;
        }

        private double DjongSummand (double xi, int i)
        {
            return i * System.Math.Pow (xi, 2);
        }
    }

    #endregion Fitness

    #region Recombination

    [Author("Lieferumfang von EvoLab")]
    [Description("Vektoraddition")]
    [Identifier("Ue2_Rekombination_Linear")]
    public class Ue2_Rekombination_Linear : EARecombination<DoubleVectorMinimize, Ue2_Population>
    {
        private IRandom rand;
        private double min, max;

        public override void Init(
            string name,
		    Ue2_Population population,
            IWorld world)
        {
            base.Init(name, population, world);

            rand = world.RandomNumberGenerator;

            min = population.min;
            max = population.max;
            if (min > max)
            {
                throw new ArgumentOutOfRangeException("Bereichs Minimum > Bereichs Maximum");
            }

            //2 Eltern, 1 Kind
            InitNumbers(2, 1);
        }

        protected override void Recombine(DoubleVectorMinimize[] parents, DoubleVectorMinimize[] children)
        {
            DoubleVectorMinimize child = children[0];
            DoubleVectorMinimize parent1 = parents[0];
            DoubleVectorMinimize parent2 = parents[1];

            double alpha = rand.NextGaussian() + 0.5d;

            for (int i = 0; i < child.vector.Length; ++i)
            {
                child.vector[i] = parent1.vector[i] * alpha + parent2.vector[i] * (1.0d - alpha);
/*                child.vector[i] = (child.vector[i] < min) ? min : child.vector[i];
                child.vector[i] = (child.vector[i] > max) ? max : child.vector[i];*/

            }
        }
    }

    #endregion Recombination

    #region Mutation

    [Author("Lieferumfang von EvoLab")]
    [Description("Mutation")]
    [Identifier("Ue2_Mutation")]

    [DoubleParameter("Mittlere Abweichung pro Gen in Prozent", 5.0d)]
    [BooleanParameter("Mutation im Gesamten Wertebereich (false = nur im Bereich der Population)", false)]
    [DoubleParameter("MUTATION_PROBABILITY", 0.8d)]

	//Temporarily deriving from IModuleMutation instead of EAMutation
    public sealed class Ue2_Mutation
    : EAMutationBase <DoubleVectorMinimize, Ue2_Population>
    {
        private IRandom rand;
        private double deviation;
        private double min, max;
        private double[] coordMin, coordMax;
        private bool mutRange;
        private double mutationProbability;

        public override void Init(
            string name,
		    Ue2_Population population,
            IWorld world)
        {
			base.Init(name, population, world);

            coordMin = new double[population.vectorLength];
            coordMax = new double[population.vectorLength];

            rand = world.RandomNumberGenerator;

            deviation = world.GetLocalDouble(name, "Mittlere Abweichung pro Gen in Prozent");
            if (!((deviation < 100) && (deviation > 0)))
                throw new ArgumentOutOfRangeException ("!((deviation <= 100) && (deviation > 0))");

            mutRange = world.GetLocalBool(name, "Mutation im Gesamten Wertebereich (false = nur im Bereich der Population)");

            min = population.min;
            max = population.max;
            if (min > max)
            {
                throw new ArgumentOutOfRangeException("Bereichs Minimum > Bereichs Maximum");
            }

            mutationProbability = world.GetLocalDouble(name, "MUTATION_PROBABILITY");

            if (mutationProbability < 0.0d || mutationProbability > 1.0d)
                throw new ArgumentOutOfRangeException(
                    "Parameter MUTATION_PROBABILITY liegt nicht zwischen 0.0 und 1.0");
        }

        private void Mutate (DoubleVectorMinimize mutant)
        {
            //TODO: Eine ZZ für alle AlLelen-dingenser, oder für jedes
            //ne eigene ???
            for (int i=0; i<mutant.vector.Length; ++i) {
                if (mutRange)
                    mutant.vector[i] += (max - min) * deviation * rand.NextGaussian() / 100.0d;
                else
                    mutant.vector[i] += (coordMax[i] - coordMin[i]) * deviation * rand.NextGaussian() / 100.0d;

                mutant.vector[i] = (mutant.vector[i]< min) ? min : mutant.vector[i];
                mutant.vector[i] = (mutant.vector[i] > max) ? max : mutant.vector[i];
            }
        }        

        public override void Run()
        {
            if (!mutRange)
                for (int i = 0; i < population.vectorLength; i++)
                {
                    coordMin[i] = population.Pop[0].vector[i];
                    coordMax[i] = population.Pop[0].vector[i];
                    for (int j = 1; j < population.Pop.Length; j++)
                    {
                        if (coordMin[i] > population.Pop[j].vector[i])
                            coordMin[i] = population.Pop[j].vector[i];
                        if (coordMax[i] < population.Pop[j].vector[i])
                            coordMax[i] = population.Pop[j].vector[i];
                    }
                }
            
			foreach (DoubleVectorMinimize ind in population.Pop) {
                if (rand.NextDouble() <= mutationProbability)
                {
                    Mutate(ind);
                }
			}

        }
    }

    #endregion Mutation

}
