/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;
using System.Text;
using System.Collections.Generic;

namespace EvoLab.Common 
{
    /// @cond
    /// <Author>WB</Author>
    /// <Revision>$Revision: 2625 $</Revision>
    /// <Id>$Id: CSVWriter.cs 2625 2008-04-25 10:16:16Z hofmandl $</Id>
    /// @endcond
    /// <summary>
    /// Der CSVWriter wird verwendet um die Hierarchie in CSV-Dateien 
    /// zu speichern.
    /// </summary>
    public class CSVWriter 
    {
        private StreamWriter sw = null;
        private char separatorChar = ';';

        /// <summary>
        /// Öffnet die im Konstruktor angegebene Datei und schreibt an ihr Ende
        /// weiteren Inhalt. Falls die Datei nicht existiert wird sie erzeugt.
        /// </summary>
        ///
        /// <param name="path">
        /// Pfad muss relativ zu dem Ordner angegeben werden, 
        /// in dem die Assembly zur Laufzeit ausgeführt wird.
        /// </param>
        ///
        /// <exception cref="System.ArgumentNullException">
        ///  Falls path eine Nullreferenz oder leer ist
        /// </exception>
        public CSVWriter(String path) 
        {
            // Überprüfe das path Argument
            if (path == null || path.Length == 0) 
            {
                throw new ArgumentNullException("[Exception] " +
                    "EvoVisAPI.Files.CSVWriter.CSVWriter() " +
                    "path ist null oder ein leerer String !");
            }

            // Verwende die Klasse FileInfo, die keine Sicherheitschecks im Gegensatz
            // zur Klasse File durchführt.
            // Der FileInfo-Konstruktor verwendet entweder den Pfad ist relativ zu 
            // dem Ordner, in dem die Assembly zur Laufzeit ausgeführt wird oder man
            // kann den voll qualifizierten Pfad zur Datei angeben.
            FileInfo fInfo = new FileInfo(path);

            try 
            {
                if (!fInfo.Exists) 
                {
                    sw = new StreamWriter(fInfo.Create());        
                } 
                else 
                {
                    // Füge ans Ende der Datei an
                    sw = fInfo.AppendText();
                }
            } 
            catch (IOException) 
            {
                this.sw = null;
            } 
            catch (Exception) 
            {
                this.sw = null;
            }
        }
        
        /// <summary>
        /// Schreibt die als Parameter übergebenen Strings in eine Zeile und
        /// schließt die Zeile dann ab. Die einzelnen Strings werden mit
        /// dem Zeichen getrennt, welches in der Membervariable separatorChar
        /// angegeben ist.
        /// </summary>
        /// <param name="cells">Eine variable Anzahl an Strings, die
        /// jeder für sich in eine Zelle der CSV Datei geschrieben wird.</param>
        public void WriteCells(params string[] cells)
        {
            int paramCount = 0;
            int i = 0;
            
            if (this.sw == null) { return; }
            
            // falls keine Parameter angegeben worden sind return
            if (cells == null) { return; }
            paramCount = cells.Length;
            if (paramCount == 0) { return; }
            
            // der letzte Parameter darf nicht mit separatorChar abgetrennt
            // werden. Schreibe daher den ersten Parameter direkt immer in 
            // die Datei ...
            this.sw.Write(cells[0]);
            
            // und nun immer mit separatorChar vorangestellt, alle anderen 
            // Elemente in Cells. So wird hinter dem letzten Element kein
            // separatorChar eingefügt
            if (paramCount > 1)
            {
                do
                {
                    i++;
                    this.sw.Write(this.separatorChar);
                    this.sw.Write(cells[i]);
                }
                while (i != (paramCount - 1));
            }
            
            // Schließe diese Zeile ab.
            this.sw.WriteLine("");
        }
        
        /// <summary>
        /// Schreibt die als Parameter übergebene Liste in eine Zeile und
        /// schließt die Zeile dann ab. Die einzelnen Strings werden mit
        /// dem Zeichen getrennt, welches in der Membervariable separatorChar
        /// angegeben ist.
        /// </summary>
        /// <param name="cells">Eine Liste von an Doubles, die
        /// in eine Zeile der CSV Datei geschrieben wird.</param>
        public void WriteListAsCells(List<Double> list)
        {
            int paramCount = 0;
            int i = 0;
            
            if (this.sw == null) { return; }
            
            // falls keine Parameter angegeben worden sind, return
            if (list == null) { return; }
            paramCount = list.Count;
            if (paramCount == 0) { return; }
            
            // der letzte Parameter darf nicht mit separatorChar abgetrennt
            // werden. Schreibe daher den ersten Parameter direkt immer in 
            // die Datei ...
            this.sw.Write(list[0]);
            
            // und nun immer mit separatorChar vorangestellt, alle anderen 
            // Elemente in Cells. So wird hinter dem letzten Element kein
            // separatorChar eingefügt
            if (paramCount > 1)
            {
                do
                {
                    i++;
                    this.sw.Write(this.separatorChar);
                    this.sw.Write(list[i]);
                }
                while (i != (paramCount - 1));
            }
            
            // Schließe diese Zeile ab.
            this.sw.WriteLine("");
        }

        /// <summary>
        /// Schließt die Datei, auf die der CSVWriter gerade zeigt.
        /// </summary>
        public void Close() 
        {
            if (this.sw == null) { return; }
            this.sw.Flush();
            this.sw.Close();
        }
    }
}
