/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

namespace EvoLab.Common
{

    /// <summary>
    /// Interface ICoreComProto
    ///
    /// Dieses Interface beschreibt das komplette Kommunikationsprotokoll
    /// fuer die Kommunikation zwischen EvoLab Client und EvoLab
    /// Server. 
    ///
    /// Das Interface wird  vom Client durch VisConnection und vom Server
    /// durch die Klasse CoreConnection implementiert. 
    ///
    /// Zusaetzlich wird das Interface von der Facade Klasse ICore
    /// implementiert, die als Schnittstelle zwischen
    /// Server-Communikationsteil und den Funktionen im Server
    /// selbst  dient. 
    ///
    /// Wenn ein Neuer Befehl fuer das Protokoll implementiert werden
    /// soll, so muss in diese Klasse eine entsprechende Methode
    /// hinzugefuegt werden. 
    /// </summary>
    /// @cond
    /// <author>Christian Fonden</author>
    /// <version>$Revision: 2638 $</version>
    /// <id>$Id: ICoreConnection.cs 2638 2008-05-08 20:38:57Z hofmandl $</id>
    /// @endcond
    public interface ICoreConnection
    {    /* {author=CF, version=0.1, since=14.04.07}*/

        // Operations
        
        /// <summary>
        /// Gibt den Status des Core zurück (Status-String
        /// <param name="strUser">Username der Verbindung</param>
        /// </summary>
        /// <returns>Objekt, das den momentanen Zustand des Core beschreibt.</returns>
        EvoLab.Common.CoreStatus GetStatus();

        /// <summary>
        /// Gibt eine Liste mit Modulen zurück, die momentan beim Core registriert sind
        /// <param name="strUser">Username der Verbindung</param>
        /// </summary>
        /// <returns>Liste der verfügbaren Module.</returns>
        EvoLab.Common.ModuleInfo.UnpreparedModuleInfo[] GetModules();

        /// <summary>
        /// Gibt den Status einer Simulation zurück, die im Core geladen ist. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strSimId">ID Der Simulation, deren Status abfragt wird</param>
        /// <returns>EvoLab.Common.SimulationStatus für die angegebene Simulation</returns>
        EvoLab.Common.SimulationStatus GetSimStatus(string strSimId);

        /// <summary>
        /// Gibt eine Liste mit allen Simulationen zurück, die momentan im Core geladen sind. 
        /// <param name="strUser">Username der Verbindung</param>
        /// </summary>
        /// <returns>Array aus SimulationInfo Objekten</returns>
        EvoLab.Common.SimulationInfo[] GetSimulations();

        /// <summary>
        /// Gibt Information zur angegebenen Simulation zurück.
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strSimId">Simulations-ID</param>
        /// </summary>
        /// <returns>SimulationInfo Objekt</returns>
        EvoLab.Common.SimulationInfo GetSimulationInfo(string strSimId);

        /// <summary>
        /// Gibt eine Liste mit allen Simulationen zurück, die momentan im Core aktiven sind (RUNNING)
        /// <param name="strUser">Username der Verbindung</param>
        /// </summary>
        /// <returns>Array aus allen momentan im Core aktiven Simulationen</returns>
        EvoLab.Common.SimulationInfo[] GetRunningSimulations();

        /// <summary>
        /// Bereitet eine Simulation mit einer bestimmten XML-Konfiguration auf den Start vor
        /// <param name="strUser">Username der Verbindung</param>
        /// </summary>
        /// <returns>Eine neue Simulation, die im Core angelegt wurde und die nun konfiguriert werden soll</returns>
        EvoLab.Common.SimulationInfo Prepare(EvoLab.Common.SimConfig simconf);

        /// <summary>
        /// Startet eine Simulation im Core. Der Simulationsstart erfolgt sofort, wenn alle
        /// Randbedingungen dafür erfüllt sind (Berechtigung, Prepare...). Die Simulation
        /// muss zuvor mit Prepare vorbereitet worden sein.
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strSimId">ID der Simulation, die gestartet werden soll</param>
        /// <returns>True wenn die Simulation erfolgreich gestartet werden konnte, sonst false</returns>
        bool Start(string strSimId);

        /// <summary>
        /// stoppt eine Simulation im Core, wenn eine Simulation mit der angegebenen
        /// ID im Core existiert und gerade ausgeführt wird. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strSimId">ID der Simulation, die gestoppt werden soll</param>
        /// <returns>True wenn die Simulation erfolgreich gestoppt werden konnte, sonst false</returns>
        bool Stop(string strSimId);

        /// <summary>
        /// Pausiert eine Simulation im Core, wenn eine Simulation mit der angegebenen
        /// ID im Core existiert und gerade ausgeführt wird. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strSimId">ID der Simulation, die pausiert werden soll</param>
        /// <returns>True wenn die Simulation erfolgreich pausiert werden konnte, sonst false</returns>
        bool Pause(string strSimId);

        /// <summary>
        /// Setzt eine Simulation im Core fort, wenn eine Simulation mit der angegebenen
        /// ID im Core existiert und gerade pausiert wird. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <returns>True wenn die Simulation erfolgreich wieder fortgesetzt werden konnte, sonst false</returns>
        /// <param name="strSimId">ID der pausierten Simulation, die fortgesetzt werden soll</param>
        bool Continue(string strSimId);

        /// <summary>
        /// Löscht eine Simulation mit der angegebenen ID unwiderruflich vom Server, wenn eine
        /// solche Simulation auf dem Server existiert. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <returns>True wenn die Simulation erfolgreich vom Core gelöscht werden konnte, sonst false</returns>
        /// <param name="strSimId">ID der Simulation, die unwiderruflich aus dem Core gelöscht  werden soll</param>
        bool Delete(string strSimId);

        /// <summary>
        /// Setzt einen bestimmten Parameter einer Simulation auf den angegebenen neuen
        /// Wert, wenn eine solche Simulation im Core existiert und wenn
        /// diese einen Parameter mit dem angegebenen Namen besitzt. 
        /// </summary>
        /// <param name="strSimId">ID der Simulation, deren Parameter abgefragt werden</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="nameValue">NameValuePair Objekt</param>
        /// <returns>True wenn der Parameterwert erfolgreich gesetzt werden konnte, sonst false</returns>
        //        bool SetSimParam(string strSimId, EvoLab.Common.ModuleInfo.IParameterInfo nameValue);

        /// <summary>
        /// Gibt eine Liste von Datenquellen zurück, die zu einer Simulation gehören, 
        /// die im Core existiert. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strSimId">ID der Simulation, deren Datenquellen abfragt werden</param>
        /// <returns>Informationen über Datenquellen</returns>
        EvoLab.Common.OutputDataSetInfo[] GetDataSourceInfo(string strSimId);

        /// <summary>
        /// Gibt Datenzeilen zur angegebenen Simulation zwischen iStart und iEnd (Zeilennummern) ab. 
        /// </summary>
        /// <param name="strSimId">abzufragende Sim</param>
        /// <param name="iStart">Start-Zeilennummer</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="iEnd">End-Zeilennummer</param>
        /// <param name="strModuleIdentifier">Bezeichner des Moduls, das die Ausgabe erstellt hat</param>
        /// <returns>Array aus Ergebnisdatensätzen</returns>
        /// <remarks>ModuleIdentifier muss lt. Vasis Request vom 22.06. mnun mitgeschickt werden</remarks>
        EvoLab.Common.OutputDataSet[] GetDatasets(string strSimId, int iStart, int iEnd, string[] strModuleIdentifiers);

        /// <summary>
        /// Führt den Upload-Vorgang für eine bestimmte Datei durch, die lokal existieren muss. 
        /// Die Datei ist danach als Modul im Core verfügbar. 
        /// Wird nur ausgeführt, wenn die lokale Datei existiert und lesbar ist. 
        /// Der Core Prozess muss dazu Schreibrechte im Dateisystem besitzen. 
        /// </summary>
        /// <param name="strFilename">Dateiname einer lokalen Datei, die hochgeladen wird</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="data">Die eigentliche Datei als ByteArray</param>
        /// <param name="bIsModule">Schalter: Wenn auf true, dann wird ein Modul hochgeladen, 
        /// ansonsten eine Normale Resourcedatei...</param>
        /// <returns>True, wenn die Datei erfolgreich hochgeladen werden konnte</returns>
        bool ModUpload(string strFilename, byte[] data, bool bIsModule);

        /// <summary>
        /// Löscht eine Datei auf dem Core aus dem Modulverzeichnis. 
        /// Dazu muss die Datei existieren und der Core Prozess muss Löschrechte
        /// für das Dateisystem besitzen. 
        /// </summary>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strFilename">Dateiname einer Datei auf dem Server, die gelöscht wird</param>
        /// <returns>True, wenn die Datei erfolgreich vom Server gelöscht werden konnte</returns>
        bool ModDel(string strFilename);

        /// <summary>
        /// Beendet die Verbindung über den Kommunikationskanal 
        /// (Verbindung wird Serverseitig zurückgesetzt). 
        /// <param name="strUser">Username der Verbindung</param>
        /// </summary>
        /// <returns>True, wenn die Verbindung ohne Fehler beender werden konnte</returns>
        //bool Exit();

        /// <summary>
        /// Gibt den Wert eines Parameters einer im Core registrierten
        /// Simulation zurück, wenn dieser Parameter und diese Simulation existieren. 
        /// </summary>
        /// <param name="strSimId">ID einer Simulation, deren Parameter abgefragt werden</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="requestInfo">Name des Parameters, der abgefragt wird</param>
        /// <returns>Der Wert des angefragten Parameters</returns>
        //        EvoLab.Common.ModuleInfo.IParameterInfo GetSimParam(string strSimId, EvoLab.Common.ModuleInfo.IParameterInfo requestInfo);

        /// <summary>
        /// Gibt dem Core an, dass er eine Simulation, die zuvor mi Prepare vor-
        /// bereitet wurde, starten soll, aber dass der Start nicht erfolgen darf, 
        /// bevor eine andere simulation beendet ist. 
        /// </summary>
        /// <param name="strSimId">ID der zu startenden Simulation</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="strStartAfterSimId">ID einer bestehenden Simulation, nach der die erste Simulation gestartet werden soll</param>
        /// <returns>True, wenn die Simulation erfolgreich in die Ausführungswarteschlage eingefügt werden konnte</returns>
        bool StartEnqueued(string strSimId, string strStartAfterSimId);

        /// <summary>
        /// Gibt dem Core an, dass er eine Simulation, die zuvor mit Prepare
        /// vorbereitet wurde, zu dem angegebenen Zeitpunkt in der Zukunft starten soll. 
        /// </summary>
        /// <param name="strSimId">ID der zu startenden Simulation</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <param name="dteStartTime">Zeitpunkt, zu dem die Simulation gestartet wird</param>
        /// <returns>True, wenn die Simulation erfolgreich in die Ausführungswarteschlage eingefügt werden konnte</returns>
        bool StartTimed(string strSimId, DateTime dteStartTime);

        /// <summary>
        /// Setzt der Simulation mit der angegebenen ID eine zusätzliche
        /// selbst generierte ID. Dies ist z.B. notwendig, damit ein Client diese Simulation 
        /// einem bestimmten Lauf zuordnen kann. 
        /// </summary>
        /// <param name="strSimId">Original-ID einer Simulation</param>
        /// <param name="strCustomId">Selbst erzeugte ID für diese Simulation</param>
        /// <param name="strUser">Username der Verbindung</param>
        /// <returns>Tre, wenn die selbst erstellte ID erfolgreich gesetzt werden konnte</returns>
        bool SetCustomId(string strSimId, string strCustomId);

        /// <summary>
        /// Fährt den EvoCore regulär herunter.
        /// </summary>
        /// <param name="strUser">Der Name des aufrufenden Benutzers.</param>
        void ShutDown ();

        /// <summary>
        /// Beendet eine Simulation.
        /// </summary>
        /// <param name="strUser">Der Name des aufrufenden Benutzers.</param>
        /// <param name="strSimId">Id der Simulatin, die beendet werden soll.</param>
        /// <returns>True wenn das Beenden eingeleitet worden ist.</returns>
        /// <seealso cref="EvoLab.Core.SimulationManagment#Finish(string)"/>
        bool Finish(string strSimId);

        /// <summary>
        /// Bricht eine Simulation ab.
        /// </summary>
        /// <param name="strUser">Der Name des aufrufenden Benutzers.</param>
        /// <param name="strSimId">Die Id der Simulation, die Abgebrochen werden soll.</param>
        /// <returns>True wenn das Abbrechen eingeleitet wirden ist.</returns>
        bool Abort(string strSimId);

        /// <summary>
        /// Bricht alle Simulationen ab.
        /// </summary>
        void AbortAllSimulations();

        /// <summary>
        /// Stellt eine Simulation wiederher.
        /// </summary>
        /// <param name="strUser">Der </param>
        /// <param name="strSimId"></param>
        void Restore(string strSimId);

        /// <summary>
        /// Löscht eine Ressourcen-Datei des Benutzers.
        /// </summary> 
        /// <param name="userName">Benutzers Name, wer die Datei laedt hoch.</param>
        /// <param name="fileName">Der Name der hochzuladenden Datei
        void DeleteResource(string fileName);

        /// <summary>
        /// Gibt ein Array mit den Namen der Ressourcendateien des
        /// Benutzers zurück.
        /// </summary>
        /// <param name="userName">Benutzers Name, wer die Datei laedt hoch.</param>
        /// <returns>Array mit den Dateinamen.</returns>
        string[] GetResources();
        /// <summary>
        /// Gibt die Simulations Konfiguration zu der angegebenen
        /// Simulation zurueck, wenn diese Simulation im Server
        /// existiert. 
        ///</summary>
        /// <param name=strUser>Benutzername des verbundenen Benutzers</param>
        /// <param name=strSimId>ID der Simulation, deren aktuelle Konfiguration abgefragt werden soll</param>
        EvoLab.Common.SimConfig GetSimConfig(string strSimId);

    } /* end interface IComProto */
} /* end namespace EvoLab.Com */
