/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

using EvoLab.Common;

namespace EvoLab.Common.ModuleInfo
{
    
    /// <summary>
    /// Diese Aufzählung erlaubt es, die Core-Module nach ihrer Funktion zu gruppieren.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2636 $</version>
    /// <id>$Id: EvoModuleInfo.cs 2636 2008-04-28 23:32:45Z hofmandl $</id>
    /// @endcond
    public enum ModuleType
    {
        FITNESS,
        RECOMBINATION,
        SELECTION,
        MUTATION,
        USER_DEFINED,
        OUTPUT,
        INITIALIZATION,
        FINALIZATION,
        SIMULATION
    }

    /// <summary>
    /// Dieser struct kann verwendet werden, um die Zuordnung
    /// EvoModuleInfo -> UnpreparedModuleInfo in O(1) zu kriegen.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2636 $</version>
    /// <id>$Id: EvoModuleInfo.cs 2636 2008-04-28 23:32:45Z hofmandl $</id>
    /// @endcond
    public struct ModuleInfoIdentifier {
        public string assemblyName;
        public string className;

        public ModuleInfoIdentifier (string assemblyName, string className) {
            this.assemblyName = assemblyName;
            this.className = className;
        }
    }

    /// <summary>
    /// Ermöglicht es, die Module nach Kompatibilität zu gruppieren.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2636 $</version>
    /// <id>$Id: EvoModuleInfo.cs 2636 2008-04-28 23:32:45Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class GroupIdentifier : ICloneable<GroupIdentifier> {
        public string populationTypeName;
        public string populationName;
        public string populationDescription;

        public GroupIdentifier () {
            populationTypeName = String.Empty;
            populationName = String.Empty;
            populationDescription = String.Empty;
        }

        public GroupIdentifier (string populationName, string populationTypeName, string populationDescription) {
            this.populationTypeName = populationTypeName;
            this.populationName = populationName;
            this.populationDescription = populationDescription;
        }

        public override bool Equals (object o) {
            if (o is GroupIdentifier) {
                return (
                    ((GroupIdentifier)o).populationTypeName
                    == this.populationTypeName);
            }
            return false;
        }

        public override int GetHashCode () {
            return (populationTypeName + populationName + populationDescription).GetHashCode ();
        }

        public GroupIdentifier Clone () {
            GroupIdentifier newClone = new GroupIdentifier ();
            newClone.populationName = this.populationName;
            newClone.populationTypeName = this.populationTypeName;
            newClone.populationDescription = this.populationDescription;
            return newClone;
        }
    }
        

    /// <summary>
    /// Jedes Objekt dieser Klasse repräsentiert eine Core-Modulinstanz.
    ///
    /// Es enthält Informationen, um das Modul im Core laden zu können,
    /// die Information, um welche Modulart es sich handelt, Informationen
    /// über alle von dem Modul benötigten Paramter und zugehörige Standardwerte.
    /// Außerdem enthält es einen Identifier, der innerhalb einer Simulation
    /// eindeutig sein muss. All diese Informationen werden per Reflection aus den
    /// binär vorliegenden Modulen gewonnen.
    ///
    /// Dieses Objekt ist in EvoLab dafür gedacht, von der
    /// Visualisierung zum Core geschickt zu werden. Die
    /// Visualisierung bekommt zwar Objekte der Kindklasse
    /// UnpreparedModuleInfo, kann diese aber mittels
    /// ToEvoModuleInfo umwandeln, so dass auf den Core nicht
    /// mehr benötigte Informationen nicht mit übertragen werden müssen.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2636 $</version>
    /// <id>$Id: EvoModuleInfo.cs 2636 2008-04-28 23:32:45Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class EvoModuleInfo
    {

        //Die folgenden beiden Strings identifizieren das Modul
        //eindeutig, so dass es im Core geladen werden kann.

        //Der Name der Assembly, die das Modul enthält: Bspw. modul.dll
        public string assemblyName;
        //der FullName der Modulklasse:
        //Bspw. EvoLab.Core.StandardModules.Permutation.Initialization
        public string className;

        //Der Typ des repräsentierten Moduls.
        public ModuleType moduleType;

        //Identifiziert eindeutig ein Modul innerhalb einer Simulation.
        public string identifier;

        //Dictionaries, die die lokalen Parameter des Moduls enthalten.
        public SerializableDictionary<string, bool> booleanParameters;
        public SerializableDictionary<string, double> doubleParameters;
        public SerializableDictionary<string, int> intParameters;
        public SerializableDictionary<string, int[]> intSequenceParameters;
        public SerializableDictionary<string, string[]> stringSequenceParameters;
        public SerializableDictionary<string, string> stringParameters;
        public SerializableDictionary<string, string[]> operatorParameters;
        public SerializableDictionary<string, string> fileParameters;

        /// <summary>
        /// Kopiert die Werte der Instanz in die übergebene Instanz.
        /// </summary>
        /// <param name="pmi">das Objekt, in das die Werte kopiert
        /// werden sollen.</param>
        /// <exception cref="ArgumentNullException">pmi ist NULL</exception>
        protected void CopyValuesInto (EvoModuleInfo pmi)
        {
            if (pmi == null)
                throw new ArgumentNullException ("Werte sollen in NullObjekt kopiert werden.");

            // Werte kopieren
            pmi.assemblyName = this.assemblyName;
            pmi.className = this.className;
            pmi.moduleType = this.moduleType;
            pmi.identifier = this.identifier;

            // Dictionaries müssen ebenfalls geklont werden, sind aber
            // keine Werttypen.
            pmi.booleanParameters = SimConfig.CloneParams<bool>(this.booleanParameters);
            pmi.doubleParameters = SimConfig.CloneParams<double>(this.doubleParameters);
            pmi.intParameters = SimConfig.CloneParams<int>(this.intParameters);

            pmi.stringParameters = SimConfig.CloneStringParams(this.stringParameters);
            pmi.fileParameters = SimConfig.CloneStringParams(this.fileParameters);

            pmi.intSequenceParameters = SimConfig.CloneArrayParams<int>(this.intSequenceParameters);

            pmi.stringSequenceParameters = SimConfig.CloneStringArrayParams(this.stringSequenceParameters);
            pmi.operatorParameters = SimConfig.CloneStringArrayParams(this.operatorParameters);
        }

        /// <summary>
        /// Diese Methode klont dieses Objekt, indem zunächst eine neue Instanz dieser 
        /// Klasse angelegt wird und anschließend sämtliche Werte dieses Objekts in die 
        /// neue Instanz kopiert werden.
        /// </summary>
        /// <returns>Eine neue Instanz dieser Klasse, welche die gleichen Werte wie 
        /// dieses Objekt enthält (aber nicht dasselbe Objekt ist)</returns>
        public EvoModuleInfo Clone()
        {
            // neue Instanz erstellen
            EvoModuleInfo newClone = new EvoModuleInfo();

            this.CopyValuesInto (newClone);

            // geklontes Objekt zurückgeben
            return newClone;
        }

    }

    /// <summary>
    /// Jedes Objekt dieser Klasse repräsentiert eine Core-Modulinstanz.
    ///
    /// Es enthält zusätzlich zu den Informationen seiner Vaterklasse
    /// Autoreninformation, eine textuelle Beschreibung
    /// des Moduls und die globalen Parameter, die das Modul fordert.
    /// All diese Informationen werden per Reflection aus den
    /// binär vorliegenden Modulen gewonnen.
    /// Dieses Objekt ist in EvoLab dafür gedacht, von der
    /// Core zur Visualisierung geschickt zu werden.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2636 $</version>
    /// <id>$Id: EvoModuleInfo.cs 2636 2008-04-28 23:32:45Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class UnpreparedModuleInfo : EvoModuleInfo {

        //Autoreninformationen
        public AuthorInfo authorInfo;
        //Modulbeschreibung
        public DescriptionInfo descriptionInfo;

        //ModuleInfos, bei denen dieser Wert gleich ist gehören zur
        //selben Gruppe und sind somit kompatibel. Ist dieser Wert
        //null, so ist das Modul kompatibel zu allen anderen Modulen.
        public GroupIdentifier groupIdentifier;

        //Vom Modul geforderte globale Parameter
        public SerializableDictionary<string, bool> globalBooleanParameters;
        public SerializableDictionary<string, double> globalDoubleParameters;
        public SerializableDictionary<string, int> globalIntParameters;
        public SerializableDictionary<string, int[]> globalIntSequenceParameters;
        public SerializableDictionary<string, string[]> globalStringSequenceParameters;
        public SerializableDictionary<string, string> globalStringParameters;
        public SerializableDictionary<string, string[]> globalOperatorParameters;
        public SerializableDictionary<string, string> globalFileParameters;

        /// <summary>
        /// Kopiert die Werte der Instanz in die übergebene Instanz.
        /// </summary>
        /// <param name="umi">das Objekt, in das die Werte kopiert
        /// werden sollen.</param>
        /// <exception cref="ArgumentNullException">umi ist NULL</exception>
        protected void CopyValuesInto (UnpreparedModuleInfo umi)
        {
            base.CopyValuesInto(umi);

            umi.authorInfo = this.authorInfo;
            umi.descriptionInfo = this.descriptionInfo;

            if (this.groupIdentifier == null) {
                umi.groupIdentifier = null;
            } else {
                umi.groupIdentifier = this.groupIdentifier.Clone ();
            }

            umi.globalBooleanParameters = SimConfig.CloneParams<bool>(this.globalBooleanParameters);
            umi.globalDoubleParameters = SimConfig.CloneParams<double>(this.globalDoubleParameters);
            umi.globalIntParameters = SimConfig.CloneParams<int>(this.globalIntParameters);

            umi.globalIntSequenceParameters = 
                SimConfig.CloneArrayParams<int>(this.globalIntSequenceParameters);

            umi.globalStringSequenceParameters = 
                SimConfig.CloneStringArrayParams(this.globalStringSequenceParameters);
            umi.globalOperatorParameters =
                SimConfig.CloneStringArrayParams(this.globalOperatorParameters);

            umi.globalStringParameters = SimConfig.CloneStringParams(this.globalStringParameters);
            umi.globalFileParameters = SimConfig.CloneStringParams(this.globalFileParameters);
        }

        /// <summary>
        /// Diese Methode klont dieses Objekt, indem zunächst eine neue Instanz dieser 
        /// Klasse angelegt wird und anschließend sämtliche Werte dieses Objekts in die 
        /// neue Instanz kopiert werden.
        /// </summary>
        /// <returns>Eine neue Instanz dieser Klasse, welche die gleichen Werte wie 
        /// dieses Objekt enthält (aber nicht dasselbe Objekt ist)</returns>
        new public UnpreparedModuleInfo Clone()
        {
            // neue Instanz erstellen
            UnpreparedModuleInfo newClone = new UnpreparedModuleInfo();

            this.CopyValuesInto (newClone);

            // geklontes Objekt zurückgeben
            return newClone;
        }

        /// <summary>
        /// Diese Methode klont dieses Objekt, indem zunächst eine neue Instanz dieser 
        /// Klasse angelegt wird und anschließend sämtliche Werte dieses Objekts in die 
        /// neue Instanz kopiert werden.
        /// </summary>
        /// <returns>Eine neue Instanz dieser Klasse, welche die gleichen Werten wie 
        /// dieses Objekt enthält (aber nicht dasselbe Objekt ist)</returns>
        public EvoModuleInfo ToEvoModuleInfo()
        {
            return base.Clone ();
        }


        /// <summary>
        /// Gibt die korrespondierende UnpreparedModuleInfo zurück und
        /// null, falls keine gefunden wird.
        /// </summary>
        /// <param name="emi">Die ModuleInfo deren Vorlage gesucht wird.</param>
        /// <param name="umi">Hier wird das passende Ding rausgesucht.</param>
        /// <exception cref="ArgumentNullException">einer der
        /// Parameter ist null.</exception>
        /// <returns>Die passende ModulVorlage oder null, falls keine
        /// vorhanden ist.</returns>
        public static UnpreparedModuleInfo GetCorrespondingModuleInfo (
            ICollection<UnpreparedModuleInfo> umis, EvoModuleInfo emi)
        {
            if (emi == null)
                throw new ArgumentNullException ("emi ist null.");
            if (umis == null)
                throw new ArgumentNullException ("umis ist null.");

            UnpreparedModuleInfo matchingUmi = null;
            foreach (UnpreparedModuleInfo availableUmi in umis) {
                if ((availableUmi.assemblyName == emi.assemblyName) &&
                    (availableUmi.className == emi.className)) {
                    matchingUmi = availableUmi;
                }
            }
            return matchingUmi;
        }

        /// <summary>
        /// Gibt die korrespondierende UnpreparedModuleInfo zurück und
        /// null, falls keine gefunden wird.
        /// </summary>
        /// <param name="emi">Die ModuleInfo deren Vorlage gesucht wird.</param>
        /// <param name="umi">Hier wird das passende Ding rausgesucht.</param>
        /// <exception cref="ArgumentNullException">einer der
        /// Parameter ist null.</exception>
        /// <returns>Die passende ModulVorlage oder null, falls keine
        /// vorhanden ist.</returns>
        public static UnpreparedModuleInfo GetCorrespondingModuleInfo (
            IDictionary<ModuleInfoIdentifier, UnpreparedModuleInfo> umis, EvoModuleInfo emi)
        {
            if (emi == null)
                throw new ArgumentNullException ("emi ist null.");
            if (umis == null)
                throw new ArgumentNullException ("umis ist null.");

            UnpreparedModuleInfo matchingUmi = null;
            umis.TryGetValue (new ModuleInfoIdentifier(emi.assemblyName, emi.className), out matchingUmi);
            return matchingUmi;
        }

    }

}
