/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Threading;
using System.Runtime.InteropServices;

using Tao.OpenGl;
using Tao.Sdl;

namespace EvoLab.Common
{
    public delegate float Function3D (float x, float y);

    public struct Point3D
    {
        public Point3D (float x, float y, float z)
        {
            this.x = x; this.y = y; this.z = z;
        }

        public readonly float x;
        public readonly float y;
        public readonly float z;
    }

	public abstract class OpenGlSdlBase : IDisposable
	{
		private readonly int screenWidth = 640;
		private readonly int screenHeight = 480;
		private readonly int colorDepth = 16;

        private readonly float height = 2.0f;
        private readonly float zoom = 1.2f;

        private float leftRightAngle = 0.0f;
        private float upDownAngle = 0.0f;

        private bool drawScene;

        //TODO: Do we really need this reference to toggle fullscreen?
		private IntPtr surface;

        private int frontFacePolygonMode = Gl.GL_POINT;
        private int backFacePolygonMode = Gl.GL_POINT;
        private bool modeChanged = true;

        protected abstract void DrawScene ();

        public OpenGlSdlBase ()
        {
            //TODO: Make numerical variables user-definable
        }

        protected void StartRendering ()
        {
            drawScene = true;
            Thread loop = new Thread(MainLoop);

            //Make sure, that the managed thread uses only one OS-Thread
            loop.SetApartmentState(ApartmentState.STA);

            loop.Start ();
        }

        protected virtual void HandleKeyPress (Sdl.SDL_keysym keysym)
        {
			switch(keysym.sym) {
            case Sdl.SDLK_ESCAPE:
                Dispose ();
                break;
            case Sdl.SDLK_F1:
                ToggleFullScreen ();
                break;
            case Sdl.SDLK_F2:
                switch (frontFacePolygonMode) {
                case Gl.GL_POINT:
                    frontFacePolygonMode = Gl.GL_LINE;
                    modeChanged = true;
                    break;
                case Gl.GL_LINE:
                    frontFacePolygonMode = Gl.GL_FILL;
                    modeChanged = true;
                    break;
                case Gl.GL_FILL:
                    frontFacePolygonMode = Gl.GL_POINT;
                    modeChanged = true;
                    break;
                }                    
                break;
            case Sdl.SDLK_F3:
                switch (backFacePolygonMode) {
                case Gl.GL_POINT:
                    backFacePolygonMode = Gl.GL_LINE;
                    modeChanged = true;
                    break;
                case Gl.GL_LINE:
                    backFacePolygonMode = Gl.GL_FILL;
                    modeChanged = true;
                    break;
                case Gl.GL_FILL:
                    backFacePolygonMode = Gl.GL_POINT;
                    modeChanged = true;
                    break;
                }                    
                break;
			}
        }

        public void ToggleFullScreen ()
        {
            Sdl.SDL_WM_ToggleFullScreen(surface);
        }

		private void ResizeScene(int width, int height)
		{
            //TODO: Check parameters more elegant
			if(height == 0)
				height = 1;
			
			double ratio = (double)width / (double)height;
			
			Gl.glViewport(0, 0, width, height); //reset viewport
			Gl.glMatrixMode(Gl.GL_PROJECTION); //choose projection matrix
            Gl.glLoadIdentity(); //reset projection matrix
            Glu.gluPerspective(45.0, ratio, 0.1, 100.0); //Set window
                                                         //aspect ratio
            Gl.glMatrixMode(Gl.GL_MODELVIEW); //choose model view matrix
            Gl.glLoadIdentity(); //reset projection matrix
		}

        private void ApplyMode ()
        {
            //TODO: Make Changeable
            Gl.glShadeModel(Gl.GL_SMOOTH); //Smooth Shading
            Gl.glClearColor(0.0f, 0.0f, 0.0f, 0.5f); //Background
            Gl.glClearDepth(1.0f); //Depth Buffer
            Gl.glEnable(Gl.GL_DEPTH_TEST); //Depth Testing
            Gl.glDepthFunc(Gl.GL_LEQUAL); //Depth Test Type
            Gl.glHint(Gl.GL_PERSPECTIVE_CORRECTION_HINT, Gl.GL_NICEST); //Perspective Correction

            Gl.glPolygonMode(Gl.GL_FRONT, frontFacePolygonMode);
            Gl.glPolygonMode(Gl.GL_BACK, backFacePolygonMode);

        }

        private void MainLoop ()
        {
            //Initialization has to be in Mainloop because OpenGl uses
            //Per-Thread-Rendering-Contexts.
            //TODO: Maybe we can find a way, to transfer a rendering
            //context from one thread to another ..
            try {

                if(Sdl.SDL_Init(Sdl.SDL_INIT_VIDEO) < 0)
                    throw new ApplicationException("SDL Initialisierung fehlgeschlagen: " + Sdl.SDL_GetError());

                IntPtr ptrVideoInfo;
                if ((ptrVideoInfo = Sdl.SDL_GetVideoInfo()) == IntPtr.Zero)
                    throw new ApplicationException("Konnte Informationen über vorhandene Grafikhardware nicht abfragen: " + Sdl.SDL_GetError());

                Sdl.SDL_VideoInfo SDLVideoInfo = (Sdl.SDL_VideoInfo)Marshal.PtrToStructure(ptrVideoInfo, typeof(Sdl.SDL_VideoInfo));

                int videoFlags;
                videoFlags = Sdl.SDL_OPENGL;
                videoFlags |= Sdl.SDL_GL_DOUBLEBUFFER;
                videoFlags |= Sdl.SDL_HWPALETTE;
                videoFlags |= Sdl.SDL_RESIZABLE;

                if(SDLVideoInfo.hw_available == 1)
                    videoFlags |= Sdl.SDL_HWSURFACE;
                else
                    videoFlags |= Sdl.SDL_SWSURFACE;

                if(SDLVideoInfo.blit_hw == 1)
                    videoFlags |= Sdl.SDL_HWACCEL;

                Sdl.SDL_GL_SetAttribute(Sdl.SDL_GL_DOUBLEBUFFER, 1);

                if ((surface = Sdl.SDL_SetVideoMode(screenWidth, screenHeight, colorDepth, videoFlags)) == IntPtr.Zero)
                    throw new ApplicationException("Video Mode set failed:\n" + Sdl.SDL_GetError ());

                //TODO: Do we need this ?
				//SDLSurface = (Sdl.SDL_Surface)Marshal.PtrToStructure(surface, typeof(Sdl.SDL_Surface));

                ResizeScene(screenWidth, screenHeight);

                Sdl.SDL_Event sdlEvent;
                drawScene = true;
                while(drawScene)
                {
                    if (modeChanged) {
                        ApplyMode ();
                        modeChanged = false;
                    }

                    DrawScene ();

                    Sdl.SDL_GL_SwapBuffers();

                    Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT); //clear screen
                    Gl.glLoadIdentity(); //reset projection matrix

                    Glu.gluLookAt(0.0f, this.height, this.zoom, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);

                    Gl.glRotatef(leftRightAngle, 0.0f, 1.0f, 0.0f);
                    Gl.glRotatef(upDownAngle, 1.0f, 0.0f, 0.0f);

                    //We don't have to have max. Framerate:
                    //TODO: Allow targeting of a specific framerate
                    System.Threading.Thread.Sleep (100);

                    while(Sdl.SDL_PollEvent(out sdlEvent) == 1)
                    {
                        switch(sdlEvent.type)
                        {
                        case Sdl.SDL_VIDEORESIZE:
                            //TODO: Do we need this ?
                            if ((surface = Sdl.SDL_SetVideoMode(sdlEvent.resize.w, sdlEvent.resize.h, colorDepth, videoFlags)) == IntPtr.Zero)
                                throw new ApplicationException ("Could not get a Surface after resize: " + Sdl.SDL_GetError());

                            //TODO: Do we need this ?
                            //SDLSurface = (Sdl.SDL_Surface)Marshal.PtrToStructure(surface, typeof(Sdl.SDL_Surface));
								
                            ResizeScene(sdlEvent.resize.w, sdlEvent.resize.h);
                            break;
                        case Sdl.SDL_MOUSEMOTION:
                            //TODO: Make dragging more intuitive
                            int x,y;
                            if ((Sdl.SDL_GetMouseState (out x, out y) & Sdl.SDL_BUTTON (Sdl.SDL_BUTTON_LMASK)) > 0) {
                                Sdl.SDL_MouseMotionEvent mme = sdlEvent.motion;
                                leftRightAngle += 0.1f * mme.xrel;
                                if (leftRightAngle >= 360.0f)
                                    leftRightAngle = 0.0f;
                                upDownAngle += 0.1f * mme.yrel;
                                if (upDownAngle >= 360.0f)
                                    upDownAngle = 0.0f;
                            }
                            break;
                        case Sdl.SDL_KEYDOWN:
                            HandleKeyPress(sdlEvent.key.keysym);
                            break;
                        case Sdl.SDL_QUIT:
                            drawScene = false;
                            break;
                        }
                    }
                }
            } finally {
                Dispose ();
            }
        }

        public virtual void Dispose ()
        {
            drawScene = false;

            //TODO: Wait for LoopThread to finish.

            Sdl.SDL_Quit();
        }
    }
}
