/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace EvoLab.Common
{
    /// @cond
    /// <author>Vasilios Grigas</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: OutputDataSetInfo.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    /// <summary>
    /// Diese Klasse wird dazu verwendet zurückzugeben, welche Datensätze
    /// (inkl. Nummer und Datum) von einer bestimmten Simulation auf dem Server
    /// verfügbar sind. Die Objekte werden vom Server über die Kommunikation
    /// zum Client geschickt. Diese Klasse ist nach den Nummern sortierbar.
    /// </summary> 
    [Serializable]
    public class OutputDataSetInfo : IComparable<OutputDataSetInfo>
    {
        public String mySimID;
        public DateTime myCreateDate;
        public DateTime myChangeDate;
        public int myNumber;
        public String myOutputModulIdentifier;
    

        /// <summary>
        /// Konstruktor. Über diesen werden alle ab nun schreibgeschützten Inhalte
        /// des Datensatzes festgelegt.
        /// </summary>
        /// <param name="SimID">Die ID der Simulation, in welcher dieser Datensatz erstellt wurde.</param>
        /// <param name="date">Das Datum, zu dem dieser Datensatz auf die Festplatte geschrieben wurde.</param>
        /// <param name="number">Die Identifizierungsnummer dieses Datensatzes, welches den Datensatz Simulations-
        /// <param name="outputModulIdentifier">Identifiziert das Outputmodul, mit welchem dieser Datensatz geschrieben wurde.</param>
        public OutputDataSetInfo(String SimID, DateTime createdate, DateTime changedate, int number,
               String outputModulIdentifier)
        {

            mySimID = SimID;
            myCreateDate = createdate;
            myChangeDate = changedate;
            myNumber = number;
            myOutputModulIdentifier = outputModulIdentifier;
        }

        public OutputDataSetInfo() { }

        /// <summary>
        /// Die ID der Simulation, in welcher dieser Datensatz erstellt wurde.
        /// </summary>
        public String SimID
        {
            get { return mySimID; }
        }


        /// <summary>
        /// Das Datum, zu dem dieser Datensatz auf der Festplatte erstmalig erzeugt wurde.
        /// </summary>
        public DateTime CreationDate
        {
            get { return myCreateDate; }
        }


        /// <summary>
        /// Das Datum, zu dem dieser Datensatz das letzte mal geändert wurde.
        /// </summary>
        public DateTime LastChangeDate
        {
            get { return myChangeDate; }
        }


        /// <summary>
        /// Die Identifizierungsnummer dieses Datensatzes, welches den Datensatz Simulations-
        /// intern identifiziert.
        /// </summary>
        public int Number
        {
            get { return myNumber; }
        }


        /// <summary>
        /// Identifiziert das Outputmodul, mit welchem dieser Datensatz geschrieben wurde.
        /// </summary>
        public String OutputModulIdentifier
        {
            get { return myOutputModulIdentifier; }
        }


        /// <summary>
        /// Die Vergleichsmethode aus dem Interface IComparable(Generisch) um eine
        /// Sortierung von Objekten dieser Klasse nach Simulation, Modulidentifier und dann Nummer zu ermöglichen.
        /// </summary>
        public int CompareTo(OutputDataSetInfo o)
        {
            // zunächst wird nach SimIDs geordnet, was allerdings eine recht
            // sinnlose Angelegenheit ist, da diese SimIDs zufällig erzeugt werden.
            if (!o.SimID.Equals(this.SimID))
            {
               return (this.SimID.CompareTo(o.SimID));
            }
            // Danach wird (jetzt innerhalb einer SimID) nach OutputModuleIdentifieren sortiert.
            else if (!o.OutputModulIdentifier.Equals(this.OutputModulIdentifier))
            {
               return (this.OutputModulIdentifier.CompareTo(o.OutputModulIdentifier)); 
            }
            // zu allerletzt wird nach der Sequenznummer geordnet.
            else
            {     
               return (this.myNumber - o.Number);
            }           
        }



    }
}
