/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

namespace EvoLab.Common
{
    /// @cond
    /// <Author>Tim Schuhmacher</Author>
    /// <revision>$Revision: 2625 $</revision>
    /// <Id>$Id: simulationinfo.cs 2625 2008-04-25 10:16:16Z hofmandl $</Id>
    /// @endcond
    /// <summary>
    /// Die Klasse SimulationInfo kapselt die Daten, die über eine Simulation abgefragt werden können.
    /// </summary>
    [Serializable]
    public class SimulationInfo
    {
        /// <summary>
        /// Die Rechenzeit, die die Run-Methode des Initmoduls zur Ausführung gebraucht hat.
        /// </summary>
        public TimeSpan InitTime;

        /// <summary>
        /// Die Rechenzeit, die die Run-Methoden aller
        /// Execute-Before-Cycle-Methoden gebraucht haben.
        /// </summary>
        public TimeSpan PreSimTime;

        /// <summary>
        /// Die Rechenzeit, die die Run-Methode des Simulationsmoduls
        /// gebraucht hat, d.h. die Rechenzeit des Zyklus.
        /// </summary>
        public TimeSpan SimTime;

        /// <summary>
        /// Gibt das Erstellungsdatum zurück.
        /// </summary>
        public DateTime CreationDate = DateTime.MinValue;
     
        /// <summary>
        /// Die durchschnittliche Rechenzeit im Zyklus pro
        /// Generation.
        /// </summary>
        public TimeSpan AverageTimePerGeneration
        {
            get
            {
                return (Generation == 0) ? TimeSpan.Zero : TimeSpan.FromTicks(SimTime.Ticks / Generation);
            }
        }

        /// <summary>
        /// Zeigt an ob die Simulation fertig ist oder nach der aktuellen Generation fertig sein wird.
        /// </summary>
        public bool WillFinish = false;

        /// <summary>
        /// Zeigt an ob das automatische speichern aktiviert ist.
        /// </summary>
        public bool AutoSave = false;

        /// <summary>
        /// Zeigt an ob die Simulation am nächst möglichen Punkt gespeichert wird.
        /// </summary>
        public bool SaveWhenPossible = false;

        /// <summary>
        /// Die SimulationsId.
        /// </summary>
        public string Id = String.Empty;

        /// <summary>
        /// Eine benutzerdefinierte Id, die vom Server nicht verwendet wird.
        /// </summary>
        public string CustomId = String.Empty;

        /// <summary>
        /// Der Besitzer der Simulation.
        /// </summary>
        public string Owner = String.Empty;

        /// <summary>
        /// Der Startzeitpunkt der Simulation. Wenn dieser Wert "null" ist wurde kein Zeitpunkt angegeben.
        /// </summary>
        public DateTime StartDate = DateTime.MaxValue;

        /// <summary>
        /// Die Position der Simulation in einer Warteschlange.
        /// </summary>
        public int QueuePosition = 0;
     
        /// <summary>
        /// Die Generation in der die Informationen abgefragt werden.
        /// </summary>
        public int Generation = 0;

        /// <summary>
        /// Die Id der Simulation die beendet sein muss, damit diese Simulation startet. Wenn dieser
        /// Wert "null" ist wurde keine Simulation angegeben auf die gewartet werden muss.
        /// </summary>
        public string WaitingFor = String.Empty;

        /// <summary>
        /// Der Zustand in dem sich die Simulation befindet.
        /// </summary>
        public SimulationStatus Status = SimulationStatus.ERROR;

        /// <summary>
        /// Die Beschreibung zu der Simulation.
        /// </summary>
        public string Description = String.Empty;

        /// <summary>
        /// Ein String der Informationen zu aufgetretenen Fehlern enthält.
        /// </summary>
        public string Log = String.Empty;

        /// <summary>
        /// Zeigt an ob die Simulation pausiert wenn sie das nächste mal am Pause-Punkt vorbei kommt.
        /// </summary>
        public bool WillPause = false;

        /// <summary>
        /// Zeigt An ob die Simulation stopped wenn die das nächste mal am Stop-Punkt vorbei kommt.
        /// </summary>
        public bool WillStop = false;

        /// <summary>
        /// Diese Methode klont dieses Objekt und gibt die neue Instanz zurück.
        /// </summary>
        /// <returns>Das geklonte neue Objekt</returns>
        public SimulationInfo Clone(string newCustomId, string newId)
        {
            SimulationInfo nsi = new SimulationInfo();

            nsi.Id = newId;
            nsi.CustomId = newCustomId;
            nsi.Description = this.Description;
            nsi.CreationDate = this.CreationDate;
            nsi.Log = this.Log;
            nsi.Owner = this.Owner;
            nsi.StartDate = this.StartDate;
            nsi.Status = this.Status;
            nsi.WaitingFor = this.WaitingFor;
            nsi.WillPause = this.WillPause;
            nsi.WillStop = this.WillStop;
            nsi.WillFinish = this.WillFinish;
            nsi.AutoSave = this.AutoSave;
            nsi.SaveWhenPossible = this.SaveWhenPossible;
            nsi.QueuePosition = this.QueuePosition;
            nsi.Generation = this.Generation;
            nsi.InitTime = this.InitTime;
            nsi.PreSimTime = this.PreSimTime;
            nsi.SimTime = this.SimTime;

            return nsi;
        }

        /// <summary>
        /// Klont das Objekt.
        /// </summary>
        /// <returns>Das geklonte Objekt.</returns>
        public SimulationInfo Clone()
        {
            return Clone(this.CustomId, this.Id);
        }

        /// <summary>
        /// Vergleicht zwei SimulationInfo-Objekte anhand der Position in der Warteschlange.
        /// </summary>
        /// <param name="info1">Objekt 1.</param>
        /// <param name="info2">Objekt 2.</param>
        /// <returns></returns>
        public static int CompareByQueuePosition(SimulationInfo info1, SimulationInfo info2)
        {
            if ((info1 != null) && (info2 != null))
            {
                return info1.QueuePosition.CompareTo(info2.QueuePosition);
            }
            else if ((info1 == null) && (info2 == null))
            {
                return 0;
            }
            else if (info1 == null)
            {
                return -1;
            }
            else
            {
                return 1;
            }
        }

        /// <summary>
        /// WB: Schreibt Inhalt in die Console. Wurde zu Debugzwecken
        /// eingeführt.
        /// DH: Wird zu Testzwecken und für den CommandLineClient von
        /// mir weiterverwendet und erweitert.
        /// </summary>
        public void DumpToConsole()
        {
            Console.WriteLine("************** [SimulationInfo] ****************");
            DumpLine("SimId: ", Id);
            DumpLine("CustomID: ", CustomId);
            DumpLine("Beschreibung: ", Description);
            DumpLine("Besitzer: ", Owner);
            DumpLine("Startdatum: ", StartDate);
            DumpLine("Status: ", Status);
            DumpLine("Wartet auf: ", WaitingFor);
            DumpLine("Simulation ist zum Pausieren markiert: ", WillPause);
            DumpLine("Simulation ist zum Stoppen markiert: ", WillStop);
            DumpLine("Simulation ist zum Abschließen markiert: ", WillFinish);
            DumpLine("Simulation ist zum Speichern markiert: ", SaveWhenPossible);
            DumpLine("Automatisches Speichern bei Zustandsübergängen aktiv: ", AutoSave);
            DumpLine("Position in der Warteschlange: ", QueuePosition);
            DumpLine("Rechenzeit für Initialisierungsmodule: ", InitTime);
            DumpLine("Rechenzeit für Module vor Zyklus: ", PreSimTime);
            DumpLine("Rechenzeit für Zyklus: ", SimTime);
            DumpLine("Durchschnittliche Rechenzeit einer Generation: ", AverageTimePerGeneration);
            DumpLine("Generation: ", Generation);
            DumpLine("Log: ", Log);
            Console.WriteLine("************************************************");
        }

        private void DumpLine(string prefix, object o)
        {
            string suffix = (o == null) ? "null" : "'" + o.ToString() + "'";
            Console.WriteLine(prefix + suffix);
        }
    }
}
