/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

using EvoLab.Common;

namespace EvoLab.CoreApi
{
   
    /// <summary>
    /// Erm�glicht den Zugriff auf die Simulationsverwaltung von Modulen heraus.
    /// </summary>
    /// <param name="simId">Die Id der beendeten Simulation.</param>
    /// <param name="cancelled">false, wenn die Simulation sich selbst
    /// beendet hat, sonst true.</param>
    public delegate void SimFinishCallback (string simId, bool cancelled);

    /// <summary>
    /// Erm�glicht den Zugriff auf die Simulationsverwaltung von Modulen heraus.
    /// Die Zugriffe auf Simulationen erfolgen �ber die Serverschnittstelle.
    /// </summary>
    /// @cond
    /// <author>Tim Schuhmacher</author>
    /// <version>$Revision: 2638 $</version>
    /// <id>$Id: ISimulationControl.cs 2638 2008-05-08 20:38:57Z hofmandl $</id>
    /// @endcond
    public interface ISimulationControl
    {

        /// <summary>
        /// Registriert eine einzelne Simulation. Nach erfolgreicher
        /// Registrierung werden Ereignisse dieser Simulation gemeldet und
        /// mittels den 'Wait*'-Methoden kann auf Ereignisse dieser Simulation
        /// gewartet werden.
        /// </summary>
        /// <param name="simId">Die Id der Simulation.</param>
        void RegisterSimulation (string simId);

        /// <summary>
        /// Registriert mehrere Simulationen. Nach erfolgreicher
        /// Registrierung werden Ereignisse dieser Simulationen gemeldet und
        /// mittels den 'Wait*'-Methoden kann auf Ereignisse dieser Simulationen
        /// gewartet werden.
        /// </summary>
        /// <param name="simIds">Liste von Ids der Simulationen.</param>
        void RegisterSimulations(List<string> simIds);

        /// <summary>
        /// Wartet bis das n�chste Ereignis der angegebenen 
        /// Simulation gemeldet wird. Der Aufrufende Thread
        /// ist w�hrend des wartens blockiert.
        /// </summary>
        /// <param name="simId">Die Id der Simulation von der das Ereignis stammen soll.</param>
        /// <returns>Die Ereignis-Historie.</returns>
        Queue<SimulationEvent> WaitOne(string simId);

        /// <summary>
        /// Wartet bis das n�chste Ereignis einer beliebigen, angegebenen Simulation
        /// gemeldet wird. Der Aufrufende Thread
        /// ist w�hrend des wartens blockiert.
        /// </summary>
        /// <param name="simIds">Die Id der Simulationen von denen das Ereignis stammen soll.</param>
        /// <returns>Die Ereignis-Historie.</returns>
        Queue<SimulationEvent> WaitAny(List<string> simIds);

        /// <summary>
        /// Wartet bis zu jeder angegebenen Simulation ein
        /// Ereignis gemeldet wurde. Der Aufrufende Thread
        /// ist w�hrend des wartens blockiert.
        /// </summary>
        /// <param name="simIds">Die Id der Simulationen von denen das Ereignis stammen soll.</param>
        /// <returns>Die Ereignis-Historie.</returns>
        Queue<SimulationEvent> WaitAll(List<string> simIds);

        /// <summary>
        /// Entfernt alle Ereignisse aus der Ereignis-Historie und setzt alle
        /// WaitHandles dieser Ereignisse zur�ck. Die entfernten Ereignisse werden
        /// zur�ckgegeben.
        /// </summary>
        /// <returns>Die Ereignis-Historie.</returns>
        Queue<SimulationEvent> GetEventHistory();

        /// <summary>
        /// Gibt den Status einer Simulation zur�ck, sofern die ID im Core gespeichert ist.
        /// </summary>
        /// <param name="strSimId">ID Der Simulation, deren Status abgefragt wird.</param>
        /// <returns>Status der jeweiligen Simulation.</returns>
        /// <exception cref="EvoLab.Common.EvoLabException">Der Befehl konnte nicht
        /// wie gew�nscht durchgef�hrt werden.</exception>
        /// <exception cref="EvoLab.Common.UnknownIdException">Die Simulation ist unbekannt.</exception>
        /// <exception cref="EvoLab.Common.PermissionException">Der Zugriff wurde verweigert.</exception>
        SimulationStatus GetSimStatus(string strSimId);

        /// <summary>
        /// L�scht eine Simulation mit der angegebenen ID unwiderruflich vom Server, wenn eine
        /// solche Simulation auf dem Server existiert.
        /// </summary>
        /// <param name="strSimId">ID der Simulation, die unwiderruflich aus dem Core gel�scht  werden soll</param>
        /// <returns>True wenn die Simulation erfolgreich vom Core gel�scht werden konnte, sonst false</returns>
        /// <exception cref="EvoLabException">Der Befehl konnte nicht
        /// wie gew�nscht durchgef�hrt werden.</exception>
        bool Delete(string strSimId);

        /// <summary>
        /// Bereitet eine Simulation f�r den Start vor.
        /// </summary>
        /// <param name="simconf">Die Konfiguration der Simulation.</param>
        /// <returns>Die ID, mit der diese Simulation fortan eindeutig
        /// auf dem Server identifizierbar ist.</returns>
        SimulationInfo Prepare (SimConfig simconf);

        /// <summary>
        /// Gibt eine Liste von Datenquellen zur�ck, die zu einer Simulation geh�ren,
        /// die im Core existiert.
        /// </summary>
        /// <param name="strSimId">ID der Simulation, deren Datenquellen abfragt werden</param>
        /// <returns>Informationen zu den abrufbaren Datenquellen</returns>
        /// <exception cref="EvoLabException">Der Befehl konnte nicht
        /// wie gew�nscht durchgef�hrt werden.</exception>
        OutputDataSetInfo[] GetDataSourceInfo(string strSimId);

        /// <summary>
        /// Gibt Datenzeilen zur angegebenen Simulation zwischen iStart und iEnd (Zeilennummern) ab.
        /// </summary>
        /// <param name="strSimId">abzufragende Sim</param>
        /// <param name="iStart">Start-Zeilennummer</param>
        /// <param name="iEnd">End-Zeilennummer</param>
        /// <param name="strModuleIdentifiers">Alle OutputDataSets im angegebenen Bereich,
        /// die von irgendeinem dieser OutputModule geschrieben wurden, sollen alle 
        /// abgefragt und heruntergeladen werden.</param>
        /// <returns>Array aus Ergebnisdatens�tzen</returns>
        /// <exception cref="EvoLabException">Der Befehl konnte nicht
        /// wie gew�nscht durchgef�hrt werden.</exception>
        OutputDataSet[] GetDatasets(string strSimId, int iStart,
            int iEnd, string[] strModuleIdentifiers);

        /// <summary>
        /// Startet eine Simulation.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool Start(string strSimId);

        /// <summary>
        /// Startet eine Simulation nach dem die andere Simulation nicht
        /// mehr l�uft.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <param name="strStartAfterSimId">Die andere Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool StartAfter(string strSimId, string strStartAfterSimId);

        /// <summary>
        /// Startet eine Simulation zum angegegebenen Zeitpunkt.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <param name="startTime">Der Zeitpunkt.</param>
        /// <returns>True bei Erfolg.</returns>
        bool StartTimed(string strSimId, DateTime startTime);

        /// <summary>
        /// Pausiert die angebene Simulation.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool Pause(string strSimId);

        /// <summary>
        /// Pausiert die aufrufende Simulation.
        /// </summary>
        /// <returns>True bei Erfolg.</returns>
        bool Pause();

        /// <summary>
        /// Setzt die angegebene Simulation fort.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool Continue(string strSimId);

        /// <summary>
        /// Bricht die angegebene Simulation ab.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool Abort(string strSimId);

        /// <summary>
        /// Bricht die aufrufende Simulation ab.
        /// </summary>
        /// <returns>True bei Erfolg.</returns>
        bool Abort();

        /// <summary>
        /// Stoppt die angegebene Simulation.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool Stop(string strSimId);

        /// <summary>
        /// Stoppt die aufrufende Simulation.
        /// </summary>
        /// <returns>True bei Erfolg.</returns>
        bool Stop();

        /// <summary>
        /// Beendet die angegebene Simulation.
        /// </summary>
        /// <param name="strSimId">Die Id der Simulation.</param>
        /// <returns>True bei Erfolg.</returns>
        bool Finish(string strSimId);

        /// <summary>
        /// Beendet die aufrufende Simulation.
        /// </summary>
        /// <returns>True bei Erfolg.</returns>
        bool Finish();

    }
}
