/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Random;

namespace EvoLab.CoreApi
{

    /// <summary>
    /// Durch dieses Interface greifen die Module während einer Simulation auf die dafür
    /// vorgesehenen Funktionen der EvoLab.Core.World zu. Dieses Interface filtert dabei
    /// alle nicht von den Modulen zu verwendenden Methoden der Klasse World heraus um
    /// ein erhöhtes Maß an Information Hiding zu erreichen.
    /// Dieses Interface hat zudem auch keinen Zugriff auf die
    /// Populationsbezogenen Member der World. Diese gibt es nur durch
    /// das generische IWorld-Interface
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2670 $</version>
    /// <id>$Id: IWorld.cs 2670 2008-05-28 23:16:21Z hofmandl $</id>
    /// @endcond
    public interface IWorld
    {
		string UniqueId
		{
			get;
		}
		
        # region Zugriff auf Ressourcendateien

        IResourceAccess ResourceAccess
        {
            get;
        }

        # endregion

        # region Zugriff auf Simulations-Log

        ILogAccess LogAccess
        {
            get;
        }

        # endregion

        # region Zugriff auf die Generation

        int Generation
        {
            get;
        }

        # endregion

        # region Zugriff auf einen globalen Zufallszahlengenerator

        IRandom RandomNumberGenerator
        {
            get;
        }

        # endregion

        # region InPut-/Output-Funktionen zum globalen Parameter-Austausch

        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ Boolean.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalBool(bool value, String name);


        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ Integer.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalInteger(int value, String name);


        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ Integer-Array.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalIntegerSequence(int[] value, String name);

        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ String-Array.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalStringSequence(string[] value, String name);

        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ String-Array.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalOperatorParameter(string[] value, String name);

        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ String.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalFileParameter(string value, String name);

        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ Double.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalDouble(double value, String name);



        /// <summary>
        /// Setzt einen für alle Module auslesbaren Parameter vom Typ String.
        /// </summary> 
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetGlobalString(String value, String name);


        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ Boolean aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        Boolean GetGlobalBool(String name);


        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ Integer aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        Int32 GetGlobalInteger(String name);



        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ Integer-Array aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        Int32[] GetGlobalIntegerSequence(String name);

        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ String-Array aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        String[] GetGlobalStringSequence(String name);


        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ Module-Array aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        string[] GetGlobalOperatorParameter(String name);


        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ Double aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        Double GetGlobalDouble(String name);


        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ String aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        String GetGlobalString(String name);

        /// <summary>
        /// Ließt einen von allen Modulen schreibbaren Parameter vom Typ String aus.
        /// </summary> 
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter. == NULL, falls der Parameter nicht vorhanden ist.</returns>
        String GetGlobalFileParameter(String name);

        # endregion

        # region InPut-/Output-Funktionen zum lokalen Parameter-Austausch

        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ Boolean.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalBool(string identifier, bool value, String name);


        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ Integer.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalInteger(string identifier, int value, String name);


        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ Integer-Array.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalIntegerSequence(string identifier, int[] value, String name);

        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ String-Array.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalStringSequence(string identifier, string[] value, String name);

        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ String-Array.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalOperatorParameter(string identifier, string[] value, String name);


        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ Double.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalDouble(string identifier, double value, String name);



        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ String.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalString(string identifier, String value, String name);

        /// <summary>
        /// Setzt einen für ein bestimmtes Modul auslesbaren Parameter vom Typ String.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="value">Der Wert des Parameters.</param>
        /// <param name="name">Der Name des Parameters.</param>
        void SetLocalFileParameter(string identifier, String value, String name);

        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ Boolean aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        Boolean GetLocalBool(string identifier, String name);


        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ Integer aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        Int32 GetLocalInteger(string identifier, String name);



        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ Integer-Array aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        Int32[] GetLocalIntegerSequence(string identifier, String name);

        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ String-Array aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        String[] GetLocalStringSequence(string identifier, String name);

        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ String-Array aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        string[] GetLocalOperatorParameter(string identifier, String name);


        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ Double aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        Double GetLocalDouble(string identifier, String name);


        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ String aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        String GetLocalString(string identifier, String name);

        /// <summary>
        /// Ließt einen für ein bestimmtes Modul schreibbaren Parameter vom Typ String aus.
        /// </summary> 
        /// <param name="identifier">Der Name, über den sich das
        /// zugreifende Modul identifizieren kann.</param>
        /// <param name="name">Der Name des Parameters.</param>
        /// <returns>Der geforderte Parameter.</returns>
        String GetLocalFileParameter(string identifier, String name);

        # endregion

    }
}
