/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

namespace EvoLab.CoreApi.Individuals {

    [Serializable]
    public abstract class Vector<T, S>
        : IComparable<T>, ITransformable<T>, IFitnessReadable, ICloneable<T>
        where T : Vector<T, S>
    {
        public abstract T Clone ();

        // ~IsWorseThan
        public abstract int CompareTo (T other);

        protected double fitness;

        //TODO: make readonly and kick dpc (has to stay as long as stupid
        //xml-serialization is used)
        public S[] vector;

        protected internal Vector(int length)
        {
            vector = new S[length];
        }

        public double Fitness {
            get {
                return fitness;
            }
            set {
                fitness = value;
            }
        }

        public void UseValuesOf (T other)
        {
            this.fitness = other.fitness;
            Array.Copy (other.vector, this.vector, this.vector.Length);
        }

		public override string ToString() {
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			
			for (int i = 0; i < vector.Length - 1; ++i) {
				sb.Append(vector[i].ToString());
				sb.Append(';');
			}
			sb.Append(vector[vector.Length - 1].ToString());
			
			return sb.ToString();
		}
		
    }

    [Serializable]
    public abstract class VectorFactory<T, S> : IndividualFactory<T> where T : Vector<T, S>
    {
        protected readonly int length;

        public VectorFactory (int length) {
            this.length = length;
        }

        public abstract T allocateNewIndividual ();
    }

    [Serializable]
    public class IntVectorMinimize : Vector<IntVectorMinimize, int>, ITransformable<IntVectorMinimize> {

        public override int CompareTo (IntVectorMinimize other)
        {
            return fitness.CompareTo (other.fitness);
        }

        public override IntVectorMinimize Clone ()
        {
            IntVectorMinimize clone = new IntVectorMinimize (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal IntVectorMinimize (int length) : base (length)
        {
            fitness = Double.MaxValue;
        }
    }

    [Serializable]
    public class IntVectorMinimizeFactory : VectorFactory<IntVectorMinimize, int>
    {
        public IntVectorMinimizeFactory (int length) : base (length) {}

        public override IntVectorMinimize allocateNewIndividual ()
        {
            return new IntVectorMinimize (length);
        }
    }

    [Serializable]
    public class IntVectorMaximize : Vector<IntVectorMaximize, int> {

        public override int CompareTo (IntVectorMaximize other)
        {
            return other.fitness.CompareTo (fitness);
        }

        public override IntVectorMaximize Clone ()
        {
            IntVectorMaximize clone = new IntVectorMaximize (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal IntVectorMaximize (int length) : base (length)
        {
            fitness = Double.MinValue;
        }
    }

    [Serializable]
    public class IntVectorMaximizeFactory : VectorFactory<IntVectorMaximize, int>
    {
        public IntVectorMaximizeFactory (int length) : base (length) {}

        public override IntVectorMaximize allocateNewIndividual ()
        {
            return new IntVectorMaximize (length);
        }
    }

    [Serializable]
    public class FloatVectorMinimize : Vector<FloatVectorMinimize, float> {

        //TODO: make readonly and kick dpc (has to stay as long as stupid
        //xml-serialization is used)
        public FloatVectorMinimize() : base (0) {} //kick ASAP

        public override int CompareTo (FloatVectorMinimize other)
        {
            return fitness.CompareTo (other.fitness);
        }

        public override FloatVectorMinimize Clone ()
        {
            FloatVectorMinimize clone = new FloatVectorMinimize (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal FloatVectorMinimize (int length) : base (length)
        {
            fitness = Double.MaxValue;
        }
    }

    [Serializable]
    public class FloatVectorMinimizeFactory : VectorFactory<FloatVectorMinimize, float>
    {
        public FloatVectorMinimizeFactory (int length) : base (length) {}

        public override FloatVectorMinimize allocateNewIndividual ()
        {
            return new FloatVectorMinimize (length);
        }
    }

    [Serializable]
    public class DoubleVectorMinimize : Vector<DoubleVectorMinimize, double> {

        //TODO: make readonly and kick dpc (has to stay as long as stupid
        //xml-serialization is used)
        public DoubleVectorMinimize() : base (0) {} //kick ASAP

        public override int CompareTo (DoubleVectorMinimize other)
        {
            return fitness.CompareTo (other.fitness);
        }

        public override DoubleVectorMinimize Clone ()
        {
            DoubleVectorMinimize clone = new DoubleVectorMinimize (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal DoubleVectorMinimize (int length) : base (length)
        {
            fitness = Double.MinValue;
        }
    }

    [Serializable]
    public class DoubleVectorMinimizeFactory : VectorFactory<DoubleVectorMinimize, double>
    {
        public DoubleVectorMinimizeFactory (int length) : base (length) {}

        public override DoubleVectorMinimize allocateNewIndividual ()
        {
            return new DoubleVectorMinimize (length);
        }
    }
}
