/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.ModuleTemplates {

    public abstract class EARecombination<I, P>
        : EARecombinationBase<I, P>
        where P : EAPopulation<I>
        where I : class, ITransformable<I>
    {
        private I[] selectedparents = null;
        private I[] selectedchildren = null;

        private int numberOfParentsPerRecombination;
        private int numberOfChildrenPerRecombination;

        private I[] offspringWithDummies = null;

        protected abstract void Recombine (I[] parents, I[] children);

        public void InitNumbers (int numberOfParentsPerRecombination, int numberOfChildrenPerRecombination)
        {
            if (selectedparents != null)
                throw new InvalidOperationException ("InitNumbers must be called only once.");
            if (numberOfChildrenPerRecombination <= 0)
                throw new ArgumentOutOfRangeException ("numberOfChildrenPerRecombination must be at least 1.");
            if (numberOfParentsPerRecombination <= 0)
                throw new ArgumentOutOfRangeException ("numberOfParentsPerRecombination must be at least 1.");

            int remainder = population.λ % numberOfChildrenPerRecombination;

            if (remainder > 0) {
                offspringWithDummies = new I [population.λ + remainder];

                for (int i=population.λ; i<population.λ + remainder; ++i) {
                    offspringWithDummies[i] = population.AllocateNewIndividual ();
                }
            }

            this.numberOfChildrenPerRecombination = numberOfChildrenPerRecombination;
            this.numberOfParentsPerRecombination = numberOfParentsPerRecombination;
            selectedparents = new I [numberOfParentsPerRecombination];
            selectedchildren = new I [numberOfChildrenPerRecombination];
			
			population.RegisterModule(name, EAModuleInfo.CreateRecombinationInfo(numberOfParentsPerRecombination * population.λ));
        }

        public override void Run ()
        {
            if (selectedparents == null)
                throw new InvalidOperationException ("InitNumbers must be called during Init.");

            I[] children = population.TargetPop;
            if (offspringWithDummies != null) {
                Array.Copy (children, offspringWithDummies, children.Length);
                children = offspringWithDummies;
            }

			I[] parents = population.Pop;
			int parentIndex = 0;
            for (int j=0; j<children.Length; j+=numberOfChildrenPerRecombination) {
                int i;
                for (i=0; i<numberOfParentsPerRecombination; ++i) {
                    selectedparents[i] = parents [parentIndex];
					parentIndex ++;
                }
                for (i=0; i<numberOfChildrenPerRecombination; ++i) {
                    selectedchildren[i] = children [j+i];
                }
                Recombine (selectedparents, selectedchildren);
            }
        }
    }
}
