/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.Populations {

    [Identifier ("EAPopulation")]

    [Description(@"")]

    public class EAPopulation<I> : ModuleRegistry<EAModuleInfo>
		where I : ITransformable<I>
    {
        public readonly int μ;

        public readonly int λ;

        private readonly IndividualFactory<I> individualFactory;
		
		private readonly List<I[]> cyclePopulations = new List<I[]>();

		private I[] currentPop;
		
		private I[] currentTargetPop;
		
		private I[] plusPop;

        public I AllocateNewIndividual ()
        {
            return individualFactory.allocateNewIndividual();
        }

		public EAPopulation (int μ, int λ, IndividualFactory<I> individualFactory)
        {
            if (individualFactory == null)
                throw new ArgumentNullException ("individualFactory");
            if (μ < 1)
                throw new ArgumentOutOfRangeException ("μ < 1");
            if (λ < 0)
                throw new ArgumentOutOfRangeException ("λ < 0");

			this.μ = μ;
            this.λ = λ;
            this.individualFactory = individualFactory;
			
			int initialPopNumber = AllocateNewCyclePopulation (μ);
			
			currentPop = cyclePopulations[initialPopNumber];
			currentTargetPop = null;
			plusPop = null;
        }
		
		public I[] Pop {
			get {
				return currentPop;
			}
		}
		
		public I[] TargetPop {
			get {
				return currentTargetPop;
			}
		}
		
		public int AllocateNewCyclePopulation (int size) {
			I[] newPop = new I[size];
			cyclePopulations.Add(newPop);
			for (int i=0; i<size; ++i) {
				newPop[i] = individualFactory.allocateNewIndividual();
			}
			return cyclePopulations.Count - 1;
		}
		
		public void SetCurrentPop (int number) {
			currentPop = cyclePopulations[number];
		}
		
		public void SetCurrentTargetPop (int number) {
			currentTargetPop = cyclePopulations[number];
		}
		
		public void ResetCurrentTargetPop () {
			currentTargetPop = null;
		}
		
		public int GetPopSize (int number) {
			return cyclePopulations[number].Length;
		}
		
		public void PreparePlusArray (int offspringsize) {
			int size = μ + offspringsize;
			if (plusPop == null || plusPop.Length != size) {
				//TODO: We could reuse old individuals here
				//TODO: Would it be safe, to copy the "offspring by reference,
				//instead of having them transformed ?
				plusPop = new I[size];
				for (int i=0; i<size; ++i) {
					plusPop[i] = AllocateNewIndividual();
				}
			}
		}
		
		public void SavePopForPlusStrategy () {
			for (int i=0; i<μ; ++i) {
				plusPop[i].UseValuesOf(Pop[i]);
			}
		}
		
		public void MergeCurrentPopPlusSavedPop () {
			int i=μ; int j=0;
			while (i<plusPop.Length) {
				plusPop[i].UseValuesOf(currentPop[j]);
				++i; ++j;
			}
			currentPop = plusPop;
		}
    }
}
