/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;

namespace EvoLab.CoreApi.Problems {

    [Serializable]
    public struct Point
    {
        public Point(double x, double y)
        {
            this.x = x; this.y = y;
        }
        public readonly double x;
        public readonly double y;
    }

    [Serializable]
    public class TSPParsingException : ApplicationException {

        internal TSPParsingException (int line, string message)
            : base (String.Format (
                    "Fehler beim Einlesen der TSP-Datei in Zeile {0}: {1}", line, message))
        { /*empty*/ }
    }

    [Serializable]
    public enum EdgeWeightType {
        EUC_2D,
        EXPLICIT,
        GEO,
        CEIL_2D,
        ATT
    }

    [Serializable]
    public enum EdgeWeightFormat {
        FULL_MATRIX,
        UPPER_ROW,
        LOWER_ROW,
        UPPER_DIAG_ROW,
        LOWER_DIAG_ROW,
        UPPER_COL,
        LOWER_COL,
        UPPER_DIAG_COL,
        LOWER_DIAG_COL
    }

    [Serializable]
    public enum DisplayDataType {
        TWOD_DISPLAY,
        COORD_DISPLAY
    }

    [Serializable]
    public enum NodeCoordType {
        NO_COORDS
    }

    [Serializable]
    public class TSP {

        /* Testing purpose */
         static void Main (string [] args) {
             TSP tsp = new TSP (new StreamReader (File.OpenRead (args[0])));

             Console.WriteLine (tsp.GetDistance (new int [] {1, 2, 3, 0}));
         }

        public readonly string name;
        public readonly string comment;

        public readonly int length;

        private readonly EdgeWeightType edgeWeightType;
        private readonly EdgeWeightFormat edgeWeightFormat;
        private readonly DisplayDataType displayDataType;
        private readonly NodeCoordType nodeCoordType;

        public readonly Point[] coords;

        //We don't use a two-dimensional array here to reduce the number of Array-Bound-Checks.
        //Therefor we have to map the citypairs with f: (int, int) -> int
        //where f(x,y) = x * length + y would be the easiest solution
        public readonly double[] distances;

        //public readonly bool symmetric;

        private static string ParseHeaderStringValue (string trimmedline, int linenumber)
        {
            int colonIndex = trimmedline.IndexOf (':');
            if (colonIndex == trimmedline.Length - 1)
                return "";
            if (colonIndex == -1)
                throw new TSPParsingException (linenumber, "Fehlender Doppelpunkt");

            return (trimmedline.Substring (colonIndex + 1)).Trim ();
        }

        private static string GetNextWord (ref string s, StreamReader sr, ref int linenumber)
        {
            if (s.Trim().Equals (String.Empty)) {
                s = sr.ReadLine ();
                linenumber ++;
            }

            if (s == null || s.Trim().Equals (String.Empty))
                throw new TSPParsingException (linenumber, "Vorzeitiges Dateiende");

            s = s.Trim ();
            //FIXME: Is behaviour of String.Split (x, int, x) correct ?
            string nextword = (s.Split(new char[]{' '}, StringSplitOptions.RemoveEmptyEntries))[0];

            s = s.Remove (0, nextword.Length).Trim ();

            return nextword;
        }

        private static int ReadWeight (ref string s, StreamReader sr, ref int linenumber)
        {
            string word = GetNextWord (ref s, sr, ref linenumber);
            int number;
            try {
                number = Int32.Parse (word);
            } catch {
                throw new TSPParsingException (linenumber, "Konnte benötigtes Gewicht nicht lesen");
            }
            return number;
        }

        private void ReadCoords (ref string s, ref int linenumber, StreamReader sr)
        {
            for (int i=0; i<length; ++i) {
                if ((s = sr.ReadLine()) == null)
                    throw new TSPParsingException (linenumber, "Vorzeitiges Ende der Datei");
                string[] elems = s.Trim().Split(new char[]{' '}, StringSplitOptions.RemoveEmptyEntries);
                if (elems.Length != 3) //TODO: Tolerate more cases
                    throw new TSPParsingException (linenumber, "Ungültige Koordinatenzeile");
                int city;
                try {
                    city = Int32.Parse(elems[0]) - 1;
                } catch {
                    throw new TSPParsingException (linenumber, "Ungültige Stadtnummer: '" + elems[0]+"'");
                }
                if (city != i)
                    throw new TSPParsingException (linenumber, "Ungültige Stadtnummer: '"+elems[0]+"'. Erwarte "+i);
                System.Globalization.NumberFormatInfo nfi = new System.Globalization.NumberFormatInfo();
                nfi.NumberDecimalSeparator = ".";
                try {
                    coords[city] = new Point(Double.Parse(elems[1], nfi), Double.Parse(elems[2], nfi));
                } catch {
                    throw new TSPParsingException (linenumber, "Ungültige Koordinate");
                }
                ++linenumber;
            }
        }

        private void CalculateEuclid2dDistancesFromCoords ()
        {
            for (int i = 0; i < length; ++i) {
                for (int j = 0; j < length; ++j) {
                    distances[i * length + j] =
                    Math.Sqrt(Math.Pow((coords[i].x - coords[j].x), 2.0)
                        + Math.Pow((coords[i].y - coords[j].y), 2.0));
                }
            }
        }

        public TSP (StreamReader sr)
        {
            if (sr == null)
                throw new ArgumentNullException ("StreamReader");

            int linenumber = 0;

            //Parse Header
            length = -1;
            bool edgeWeightTypeParsed = false;
            bool edgeWeightFormatParsed = false;
            bool displayDataTypeParsed = false;
            bool nodeCoordTypeParsed = false;
            bool typeParsed = false;
            string s;
            while ((s = sr.ReadLine()) != null) {
                ++linenumber;
                s = s.Trim ();

                if (s.StartsWith ("name", StringComparison.InvariantCultureIgnoreCase)) {
                    if (name != null)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe von NAME");
                    name = ParseHeaderStringValue (s, linenumber);
                } else if (s.StartsWith ("comment", StringComparison.InvariantCultureIgnoreCase)) {
                    if (comment != null)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe von COMMENT");
                    comment = ParseHeaderStringValue (s, linenumber);
                } else if (s.StartsWith ("type", StringComparison.InvariantCultureIgnoreCase)) {
                    if (typeParsed)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe von TYPE");
                    if (!ParseHeaderStringValue (s, linenumber).Equals("tsp", StringComparison.InvariantCultureIgnoreCase))
                        throw new TSPParsingException (linenumber, "TYPE != TSP");
                    typeParsed = true;
                } else if (s.StartsWith ("dimension", StringComparison.InvariantCultureIgnoreCase)) {
                    if (length != -1)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe von DIMENSION");
                    try {
                        length = Int32.Parse (ParseHeaderStringValue (s, linenumber));
                    } catch {
                        throw new TSPParsingException (linenumber, "Konnte DIMENSION nicht einlesen");
                    }
                    if (length < 1)
                        throw new TSPParsingException (linenumber, "DIMENSION zu klein");
                } else if (s.StartsWith ("edge_weight_type", StringComparison.InvariantCultureIgnoreCase)) {
                    if (edgeWeightTypeParsed)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe von EDGE_WEIGHT_TYPE");
                    s = ParseHeaderStringValue (s, linenumber);
                    edgeWeightTypeParsed = true;

                    try {
                        edgeWeightType = (EdgeWeightType) Enum.Parse (typeof (EdgeWeightType), s, false);
                    } catch {
                        throw new TSPParsingException (linenumber, "Unbekannter Wert für EDGE_WEIGHT_TYPE: '"+ s + "'");
                    }

                } else if (s.StartsWith ("edge_weight_format", StringComparison.InvariantCultureIgnoreCase)) {
                    if (edgeWeightFormatParsed)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe des EDGE_WEIGHT_FORMAT");
                    s = ParseHeaderStringValue (s, linenumber);
                    edgeWeightFormatParsed = true;

                    try {
                        edgeWeightFormat = (EdgeWeightFormat)Enum.Parse (typeof (EdgeWeightFormat), s, false);
                    } catch {
                        throw new TSPParsingException (linenumber, "Unbekannter Wert für EDGE_WEIGHT_FORMAT");
                    }

                } else if (s.StartsWith ("edge_data_format", StringComparison.InvariantCultureIgnoreCase)) {
                    //TODO:
                } else if (s.StartsWith ("display_data_type", StringComparison.InvariantCultureIgnoreCase)) {
                    if (displayDataTypeParsed)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe des DISPLAY_DATA_TYPE");
                    displayDataTypeParsed = true;
                    s = ParseHeaderStringValue (s, linenumber);

                    try {
                        displayDataType = (DisplayDataType)Enum.Parse (typeof (DisplayDataType), s, false);
                    } catch {
                        throw new TSPParsingException (linenumber, "Unbekannter Wert für DISPLAY_DATA_TYPE");
                    }
                } else if (s.StartsWith ("node_coord_type", StringComparison.InvariantCultureIgnoreCase)) {
                    if (nodeCoordTypeParsed)
                        throw new TSPParsingException (linenumber, "Mehrfache Angabe des NODE_COORD_TYPE");
                    nodeCoordTypeParsed = true;
                    s = ParseHeaderStringValue (s, linenumber);

                    try {
                        nodeCoordType = (NodeCoordType)Enum.Parse (typeof (NodeCoordType), s, false);
                    } catch {
                        throw new TSPParsingException (linenumber, "Unbekannter Wert für NODE_COORD_TYPE");
                    }
                } else {
                    break;
                }
            }
            if (!typeParsed)
                throw new TSPParsingException (linenumber, "Kein TYPE angegeben");
            if (!edgeWeightTypeParsed)
                throw new TSPParsingException (linenumber, "Kein EDGE_WEIGHT_TYPE angegeben");
            if (length == -1)
                throw new TSPParsingException (linenumber, "Fehlende Angabe DIMENSION im Header");

            //Parse Body
            do {
                s = s.Trim ();
                switch (s)
                {
                case "EOF":
                    switch (edgeWeightType)
                    {
                    case EdgeWeightType.EUC_2D:
                        if (coords == null)
                            throw new TSPParsingException (linenumber, "EUC_2D benötigt eine NODE_COORD_SECTION");
                        distances = new double [length * length];
                        CalculateEuclid2dDistancesFromCoords ();
                        break;
                    case EdgeWeightType.EXPLICIT:
                        if (distances == null)
                            throw new TSPParsingException (linenumber, "EXPLICIT benötigt eine EDGE_WEIGHT_SECTION");
                        break;
                    default:
                        throw new TSPParsingException (linenumber, "EDGE_WEIGHT_TYPE wird nicht unterstützt: '" + edgeWeightType + "'");
                    }
                    return;
                case "NODE_COORD_SECTION":
                    if (coords != null)
                        throw new TSPParsingException (linenumber, "Es wurde bereits eine NODE_COORD_SECTION oder eine DISPLAY_DATA_SECTION eingelesen");
                    coords = new Point[length];
                    ReadCoords (ref s, ref linenumber, sr);
                    break;
                case "EDGE_WEIGHT_SECTION":
                    if (distances != null)
                        throw new TSPParsingException (linenumber, "Es wurde bereits eine EDGE_WEIGHT_SECTION eingelesen");
                    distances = new double[length * length];

                    linenumber++;
                    if ((s = sr.ReadLine ()) == null)
                        throw new TSPParsingException (linenumber, "Vorzeitiges Dateiende. Erwarte Kantengewichte");

                    switch (edgeWeightFormat)
                    {
                    case EdgeWeightFormat.FULL_MATRIX:
                        for (int i=0; i<length; ++i) {
                            for (int j=0; j<length; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                            }
                        }
                        break;
                    case EdgeWeightFormat.UPPER_ROW:
                        for (int i=0; i<length; ++i) {
                            for (int j=i+1; j<length; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                            distances [i * length + i] = 0;
                        }
                        break;
                    case EdgeWeightFormat.LOWER_ROW:
                        for (int i=1; i<length; ++i) {
                            for (int j=0; j<=i; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                            distances [i * length + i] = 0;
                        }
                        break;
                    case EdgeWeightFormat.UPPER_DIAG_ROW:
                        for (int i=0; i<length; ++i) {
                            for (int j=i; j<length; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                        }
                        break;
                    case EdgeWeightFormat.LOWER_DIAG_ROW:
                        for (int i=0; i<length; ++i) {
                            for (int j=0; j<=i; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                        }
                        break;
                    case EdgeWeightFormat.UPPER_COL:
                        for (int i=1; i<length; ++i) {
                            for (int j=0; j<=i; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                            distances [i * length + i] = 0;
                        }
                        break;
                    case EdgeWeightFormat.LOWER_COL:
                        for (int i=0; i<length; ++i) {
                            for (int j=i+1; j<length; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                            distances [i * length + i] = 0;
                        }
                        break;
                    case EdgeWeightFormat.UPPER_DIAG_COL:
                        for (int i=0; i<length; ++i) {
                            for (int j=0; j<=i; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                        }
                        break;
                    case EdgeWeightFormat.LOWER_DIAG_COL:
                        for (int i=0; i<length; ++i) {
                            for (int j=i; j<length; ++j) {
                                int weight = ReadWeight (ref s, sr, ref linenumber);
                                distances [i * length + j] = weight;
                                distances [j * length + i] = weight;
                            }
                        }
                        break;
                    }
                    break;
                case "DISPLAY_DATA_SECTION":
                    if (coords != null)
                        throw new TSPParsingException (linenumber, "Es wurde bereits eine NODE_COORD_SECTION oder eine DISPLAY_DATA_SECTION eingelesen");
                    coords = new Point[length];
                    ReadCoords (ref s, ref linenumber, sr);
                    break;
                default:
                    throw new TSPParsingException (linenumber, "Sektions-Schlüsselwort erwartet. Fand: '" + s + "'");
                }
            } while ((s = sr.ReadLine ()) != null);

            throw new TSPParsingException (linenumber, "Vorzeitiges Ende der Datei");
        }

        public double GetDistance(int c1, int c2)
        {
            return distances[c1 * length + c2];
        }

        public double GetDistance(int[] perm)
        {
            if (perm == null)
                throw new ArgumentNullException ("perm");
            if (perm.Length != length)
                throw new ArgumentException ("Permutation has wrong length");

            double tourLength = distances[perm[perm.Length - 1] * length + perm[0]];

            for (int i = 0; i < perm.Length - 1; i++) {
                tourLength += distances[perm[i] * length + perm[i+1]];
            }
            return tourLength;
        }
    }
}
