/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;
using System.Xml.Serialization;
using System.Collections.Generic;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace EvoLab.Core.StandardModules.Universal
{

    [Author("Lieferumfang von EvoLab")]
    [Description("Standard-Fitnessausgabe für die Analyse: Alle Fitnesswerte")]
    [IntParameter("INTERVAL", 1)]
    [Identifier("EA_OutputAllFitnessValues")]
    public class EAOutputFitnessValues<I, P> : EAOutputBase<I, P>
		where P : EAPopulation<I>
        where I : IFitnessReadable, ITransformable<I>
    {
        private int myInterval;
        private int myLastWrittenPopulation;

		private readonly XmlSerializer serializer = new XmlSerializer (typeof(byte[]));
		
        /// <summary>
        /// Diese Methode muss vor dem Verwenden dieses Moduls aufgerufen werden.
        /// Sie weißt dem Modul alle benötigten Parameter zu.
        /// </summary> 
        /// <param name="MyName">Der String, unter welchem das Modul innerhalb einer Simulation
        /// identifiziert werden kann.</param>
        /// <param name="OurWorld">Das World-Objekt, das für die dieses Modul verwendende Simulation
        /// verwendet wird.</param>
        /// <param name="fileAccess">Das OutputAccess-Objekt, mit welchem dieses Outputmodul Datensätze
        /// auf die Festplatte schreiben kann.</param>
        public override void Init(string name, P population, IWorld world, IOutputAccess fileAccess)
        {
			base.Init(name, population, world, fileAccess);
  
            // Parameter INTERVAL wird eingelesen und kontrolliert:
            myInterval = world.GetLocalInteger(name,"INTERVAL");
            if (myInterval < 1)
            {
                throw new ArgumentOutOfRangeException("Parameter 'INTERVAL' < 1.");
            }

            // Damit beim 1. Mal ein Output stattfindet:
            myLastWrittenPopulation = -myInterval;
        }


        /// <summary>
        /// Dies ist die Methode, welche innerhalb des Modul-Zyklus während einer Simulation in einer 
        /// Schleife aufgerufen wird. Sie schreibt alle INTERVAL Male alle Fitnesswerte in einen Datensatz.
        /// </summary> 
        public override void Run()
        {
            if ((world.Generation - myLastWrittenPopulation) >= myInterval)
            {
				int fitnessCount = population.Pop.Length;
				byte[] fitnessValues = new byte[(fitnessCount + 1) * 8];
                System.BitConverter.GetBytes(world.Generation).CopyTo(fitnessValues, 0);
                System.BitConverter.GetBytes(fitnessCount).CopyTo(fitnessValues, 4);

                for (int i = 0; i < population.Pop.Length; i++)
                {
                    System.BitConverter.GetBytes(population.Pop[i].Fitness).CopyTo(fitnessValues,(i+1)*8);
                }

				StringWriter sw = new StringWriter ();
				serializer.Serialize (sw, fitnessValues);
				
                fileAccess.WriteDataSet( sw.ToString(), 16, name);
				
				myLastWrittenPopulation = world.Generation;
			}
        }
    }
}
