/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using System.Collections.Generic;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace EvoLab.Core.StandardModules.Universal {

	//TODO: Check for at least two selections or plus strategy 
	
	[Identifier("EA_Simulation")]
	
	[BooleanParameter("PLUS_STRATEGY", false)]
	
    [OperatorParameter("OPERATOR_MODULES", new string[] { })]

    public class EASimulation<I, P> : IModuleSimulation<P> where I : ITransformable<I> where P : EAPopulation<I>
    {
		interface CycleItem {
			void Run ();
		}
		
		class PlusSaveItem : CycleItem {
			private readonly P population;
			
			public PlusSaveItem (P population) {
				if (population == null)
					throw new ArgumentNullException("population");
				this.population = population;
			}
			
			public void Run () {
				population.SavePopForPlusStrategy();
			}
		}
		
		class ModuleItem : CycleItem {
			protected readonly IModuleCore moduleInstance;
			
			public ModuleItem (IModuleCore moduleInstance) {
				if (moduleInstance == null)
					throw new ArgumentNullException ("moduleInstance");
				this.moduleInstance = moduleInstance;				
			}
			public virtual void Run () {
				moduleInstance.Run ();
			}
		}
		
		class PopSizeChangingModuleItem : ModuleItem {
			protected readonly int popId;
			protected readonly P population;
			
			public PopSizeChangingModuleItem (IModuleCore moduleInstance, P population, int popId) : base(moduleInstance) {
				if (population == null)
					throw new ArgumentNullException("population");
				this.population = population;
				this.popId = popId;
			}
			
			public override void Run () {
				population.SetCurrentTargetPop(popId);
				moduleInstance.Run();
				population.ResetCurrentTargetPop();
				population.SetCurrentPop(popId);
			}
		}
		
		class PlusSelectionModuleItem : PopSizeChangingModuleItem {
			public PlusSelectionModuleItem (IModuleCore moduleInstance, P population, int popId)
			: base(moduleInstance, population, popId) {}
			
			public override void Run () {
				population.SetCurrentTargetPop(popId);
				population.MergeCurrentPopPlusSavedPop ();
				moduleInstance.Run();
				population.ResetCurrentTargetPop();
				population.SetCurrentPop(popId);
			}
		}

        protected IWorld world;
        protected P population;
        protected IModuleLoader moduleLoader;
        protected string name;

        private List<CycleItem> cycle;
		
		private bool plusStrategy;

		private readonly Dictionary<string, IModuleCore> loadedModules
			= new Dictionary<string, IModuleCore>();
		
		protected IModuleCore GetModule (string identifier) {
			IModuleCore module;
			if (!loadedModules.TryGetValue(identifier, out module)) {
				module = moduleLoader.LoadModule(identifier);
				loadedModules.Add(identifier, module);
			}
			return module;
		}
		
        public virtual void Init(
            string name,
            P population,
		    IWorld world,
		    IList<string> preCycleModules,
            IModuleLoader moduleLoader)
        {
            if (name == null)
                throw new ArgumentNullException ("name");
            if (population == null)
                throw new ArgumentNullException ("population");
            if (world == null)
                throw new ArgumentNullException ("world");
            if (moduleLoader == null)
                throw new ArgumentNullException ("moduleLoader");
            if (preCycleModules == null)
                throw new ArgumentNullException ("preCycleModules");

            this.name = name;
            this.world = world;
            this.moduleLoader = moduleLoader;
            this.population = population;

            foreach (string module in preCycleModules) {
                IModuleCore mod = GetModule (module);
				ModuleType type = population.GetRegistrationData(module).type;
                if (type == ModuleType.SELECTION || type == ModuleType.RECOMBINATION) {
                    throw new ArgumentException ("Precyclemodules may not change the size of the population. Selection and Recombination are therefor not allowed.");
                }
                mod.Run ();
            }

			plusStrategy = world.GetLocalBool(name, "PLUS_STRATEGY");
			
            string[] operator_modules = world.GetLocalOperatorParameter(name, "OPERATOR_MODULES");

			PrepareCycle(operator_modules);
		}

		private void PrepareCycle (string[] operators)
		{
			cycle = new List<CycleItem> (operators.Length);

			if (plusStrategy) {
				cycle.Add(new PlusSaveItem(population));
			}
			
			List<IModuleCore> moduleInstances = new List<IModuleCore>(operators.Length);
			foreach (string module in operators) {
				moduleInstances.Add(GetModule(module));
			}

			EAModuleInfo nextPopSizeChangingModule = null;
			string nextPopSizeChangingModuleIdentifier = null;
			IModuleCore moduleInstance = null;
			string moduleIdentifier = null;
			EAModuleInfo moduleRegistrationData = null;
			int currentPopId = 0;
			int currentPopsize = population.μ;

			for (int i=0; i<operators.Length; ++i) {
				moduleIdentifier = operators[i];
				moduleInstance = moduleInstances[i];
				moduleRegistrationData = population.GetRegistrationData(moduleIdentifier);
				
				if (moduleRegistrationData.type == ModuleType.SELECTION) {
					if (!GetNextPopSizeChangingModule (operators, i+1, out nextPopSizeChangingModuleIdentifier, out nextPopSizeChangingModule)) {
						//We are the last popsize changing module in the cycle
						currentPopId = 0;
						currentPopsize = population.GetPopSize(currentPopId);
						if (plusStrategy) {
							population.PreparePlusArray(currentPopsize);
							cycle.Add(new PlusSelectionModuleItem (moduleInstance, population, currentPopId));
						} else {
							cycle.Add(new PopSizeChangingModuleItem (moduleInstance, population, currentPopId));
						}
					} else {
						//We have a sucessor
						if (nextPopSizeChangingModule.sourcePopSizeType == PopSizeType.Specific) {
							//Our successor needs a specific popsize
							currentPopsize = nextPopSizeChangingModule.sourcePopSize;
							currentPopId = population.AllocateNewCyclePopulation(currentPopsize);
							cycle.Add(new PopSizeChangingModuleItem (moduleInstance, population, currentPopId));
						} else {
							//Our successor is happy with every input pop size so stay at the same pop size
							currentPopId = population.AllocateNewCyclePopulation(currentPopsize);
							cycle.Add(new PopSizeChangingModuleItem (moduleInstance, population, currentPopId));
						}
					}
				} else if (moduleRegistrationData.type == ModuleType.RECOMBINATION) {
					if (moduleRegistrationData.sourcePopSizeType == PopSizeType.Specific && currentPopsize != moduleRegistrationData.sourcePopSize) {
						throw new Exception ("Module '"+moduleIdentifier+"' requires specific population size, but no selection takes place beforehand.");
					}
					if (GetNextPopSizeChangingModule (operators, i+1, out nextPopSizeChangingModuleIdentifier, out nextPopSizeChangingModule)) {
						if (nextPopSizeChangingModule.type == ModuleType.RECOMBINATION) {
							throw new Exception ("Module '"+moduleIdentifier+"' and '"+nextPopSizeChangingModuleIdentifier+"' must have a Selection module between them");
						}
						currentPopsize = population.λ;
						currentPopId = population.AllocateNewCyclePopulation(currentPopsize);
						cycle.Add(new PopSizeChangingModuleItem (moduleInstance, population, currentPopId));
					} else {
						throw new Exception ("Module '"+moduleIdentifier+"' has no succeeding selection module.");
					}
				} else {
					cycle.Add(new ModuleItem (moduleInstance));
				}
			}
			
		}
		
		private bool GetNextPopSizeChangingModule (string[] identifiers, int fromIndex, out string identifier, out EAModuleInfo info)
		{
			identifier = null;
			info = null;

			for (int i=fromIndex; i<identifiers.Length; ++i) {
				identifier = identifiers[i];
				info = population.GetRegistrationData(identifier);
				
				if (info.type == ModuleType.SELECTION || info.type == ModuleType.RECOMBINATION) {
					return true;
				}
			}
			return false;
		}

        public virtual void Run()
        {
            foreach (CycleItem item in cycle)
            {
                item.Run();
            }
        }
		
		public void Dispose () {}

    }

}
