/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.ModuleTemplates;

namespace EvoLab.Core.StandardModules.Universal
{

	[Identifier("EA_Permutation_MutationExchange")]
	
    [IntParameter("MIN_N", 2)]
    [IntParameter("MAX_N", 2)]

    public class PermutationMutationExchange<I, P> : EAMutation<I, P> where P : VectorPopulation<I, int> where I : Vector<I, int>
    {
        protected int vectorLength;

        private int minN;
        private int maxN;

        private int[] mutatedPositions; // Speichert die mutierten Positionen
        private int[] mutatedValues; // Speichert die mutierten Werte
        
        public override void Init(String name, P population, IWorld world)
        {
            base.Init(name, population, world);

            vectorLength = population.vectorLength;

            minN = world.GetLocalInteger(name, "MIN_N");
            if ((minN < 2) || (minN > vectorLength ))
                throw new ArgumentOutOfRangeException("Parameter 'MIN_N' < 2 or > DIMENSION");

            maxN = world.GetLocalInteger(name, "MAX_N");
            if ((maxN < 2) || (maxN < minN) || (maxN > vectorLength) )
                throw new ArgumentOutOfRangeException("Parameter 'MAX_N' < 2 or < 'MIN_N' or > vectorLength.");

            mutatedPositions = new int[maxN];
            mutatedValues = new int[maxN];
        }

        protected override void Mutate(I mutant)
        {
            int count = 0;
            if (minN == maxN)
                count = minN;
            else
                count = world.RandomNumberGenerator.Next(minN, maxN + 1);

            // Die zu mutierenden Positionen werden ermittelt:
            int nextPos;
            int oldPos = -1;
            for (int i = 0; i < count; i++)
            {
                nextPos = world.RandomNumberGenerator.Next(oldPos + 1, vectorLength - count + i + 1);
                mutatedPositions[i] = nextPos;
                mutatedValues[i] = mutant.vector[nextPos];
                oldPos = nextPos;
            }

            // Die Positionen werden neu geschrieben:
            mutant.vector[mutatedPositions[0]] = mutatedValues[count - 1];
            for (int i = 1; i < count; i++)
            {
                mutant.vector[mutatedPositions[i]] = mutatedValues[i-1];
            }
        }
    }
}
